<?php
namespace Targetbay\Tracking\Block;
use Targetbay\Tracking\Api\ListProductInterface;

class Tracking extends \Magento\Framework\View\Element\Template {

	/**
	 * @var \Targetbay\Tracking\Helper\Data
	 */
	protected $trackingHelper;

	/**
	 * @var \Targetbay\Tracking\Helper\Inventory
	 */
	protected $trackingInventoryHelper;

	/**
	 * @var \Magento\Customer\Model\Session\Proxy
	 */
	protected $customerSession;

	/**
	 * @var \Magento\Framework\Stdlib\CookieManagerInterface
	 */
	protected $cookieManager;

	/**
	 * @var \Magento\Catalog\Model\ProductFactory
	 */
	protected $productFactory;

	/**
	 * @param \Magento\Framework\Registry $registry
	 * @param \Targetbay\Tracking\Helper\Data $trackingHelper
	 * @param \Targetbay\Tracking\Helper\Inventory $trackingInventoryHelper
	 * @param \Magento\Checkout\Model\Session\Proxy $checkoutSession
	 * @param \Magento\Customer\Model\Session\Proxy $customerSession
	 * @param \Magento\Framework\Stdlib\CookieManagerInterface $cookieManager
	 * @param \Magento\CatalogInventory\Model\Stock\StockItemRepository $stockItemRepository
	 * @param \Magento\Catalog\Model\ProductFactory $productFactory
	 */
	public function __construct(\Magento\Catalog\Model\ProductFactory $productFactory, \Magento\Framework\View\Element\Template\Context $context, \Targetbay\Tracking\Helper\Data $trackingHelper, \Targetbay\Tracking\Helper\Inventory $trackingInventoryHelper, \Magento\Customer\Model\Session\Proxy $customerSession, \Magento\Framework\Stdlib\CookieManagerInterface $cookieManager) {
		parent::__construct($context);
		$this -> _productFactory = $productFactory;
		$this -> trackingHelper = $trackingHelper;
		$this -> trackingInventoryHelper = $trackingInventoryHelper;
		$this -> cookieManager = $cookieManager;
		$this -> customerSession = $customerSession;
	}

	/**
	 * Get block cache life time
	 *
	 * @return int
	 */
	// protected function getCacheLifetime() {
	// return 0;
	// }

	/**
	 * @return array
	 */
	public function getProductInfo() {
		$pId = $this -> getRequest() -> getParam('id');
		$action = $this -> getRequest() -> getFullActionName();
		if ($action == 'checkout_cart_configure' || $action == 'wishlist_index_configure') {
			$pId = $this -> getRequest() -> getParam('product_id');
		}
		$product = $this -> _productFactory -> create() -> load($pId);
		$productInfo['product_name'] = $product -> getName();
		$productInfo['product_id'] = $product -> getId();
		$productInfo['product_image'] = $this -> trackingHelper -> getImageUrl($product, 'image');
		$productInfo['product_url'] = $product -> getUrlModel() -> getUrl($product);
		return $productInfo;
	}

	/**
	 * @return array
	 */
	public function getUserInfo() {
		if ($this -> customerSession -> isLoggedIn()) {
			$userInfo['user_id'] = $this -> customerSession -> getId();
			$userInfo['user_email'] = $this -> customerSession -> getCustomer() -> getEmail();
			$userInfo['user_name'] = $this -> customerSession -> getCustomer() -> getName();
			return $userInfo;
		}
		//For guest user
		$visitorName = \Targetbay\Tracking\Helper\Data::ANONYMOUS_USER;
		$userInfo['user_id'] = $this -> cookieManager -> getCookie('targetbay_session_id');
		$userInfo['user_email'] = '';
		$userInfo['user_name'] = $visitorName;
		return $userInfo;
	}

	/**
	 * @return int|string
	 */
	public function getProductStockInfo() {
		if ($this -> trackingInventoryHelper -> getInventryStatus() == 1) {
			return $this -> trackingInventoryHelper -> getProductInfo();
		}
		return false;
	}

	/**
	 * @return bool|int
	 */
	public function getStockAvailability() {
		$backOrderStatus = $this -> trackingHelper -> getBackorderStatus();
		if ($backOrderStatus)
			return $this -> trackingInventoryHelper -> getInventryStatus();
		return false;
	}

}
