<?php

namespace Targetbay\Tracking\Controller\Index;

use Magento\Framework\App\Action\Context;
use Magento\Framework\Controller\ResultFactory;

class TbAddtobag extends \Magento\Framework\App\Action\Action
{

    /**
     * @var \Magento\Checkout\Model\Cart
     */
    protected $cart;

    /**
     * @var \Magento\Catalog\Model\Product
     */
    protected $productFactory;

    /**
     * @var \Targetbay\Tracking\Helper\Inventory
     */
    protected $trackingInventoryHelper;

    /**
     * @var \Magento\Store\Model\StoreManagerInterface
     */
    protected $storeManager;

    /**
     * @var \Psr\Log\LoggerInterface
     */
    public $logger;

    /**
     * @param \Magento\Framework\App\Action\Context $context
     * @param \Magento\Checkout\Model\Cart $cart
     * @param \Magento\Catalog\Model\Product $productFactory
     * @param \Targetbay\Tracking\Helper\Inventory $trackingInventoryHelper
     * @param \Magento\Store\Model\StoreManagerInterface $storeManager
     * @param \Psr\Log\LoggerInterface $logger
     */
    public function __construct(
        Context $context,
        \Magento\Checkout\Model\Cart $cart,
        \Magento\Catalog\Model\Product $productFactory,
        \Targetbay\Tracking\Helper\Inventory $trackingInventoryHelper,
        \Magento\Store\Model\StoreManagerInterface $storeManager,
        \Psr\Log\LoggerInterface $logger
    ) {
        parent::__construct($context);
        $this->cart = $cart;
        $this->productFactory = $productFactory;
        $this->trackingInventoryHelper = $trackingInventoryHelper;
        $this->storeManager = $storeManager;
        $this->logger = $logger;
    }

    /**
     * Reload product to shopping cart
     *
     * @return \Magento\Framework\Controller\Result\Redirect
     */
    public function execute()
    {
        $resultRedirect = $this->resultFactory->create(ResultFactory::TYPE_REDIRECT);

        $utmSource = $this->getRequest()->getParam('utm_source');
        $utmToken = $this->getRequest()->getParam('token');
        $productData = base64_decode($this->getRequest()->getParam('product_id'));

        $staticArrayDecode = json_decode($productData, true);

        $productIds = $staticArrayDecode;
        $storeId = $this->storeManager->getStore()->getId();
        $qty = 1;

        $resultRedirect->setPath('checkout/cart', ['utm_source' => $utmSource, 'token' => $utmToken]);

        //$resultRedirect->setPath('checkout/cart');

        if (count($staticArrayDecode) < 1) {
            return $resultRedirect;
        }

        try {
            $bundleOptionQty = [];

            foreach ($staticArrayDecode as $key => $value) {
                $productId = $key;
                $productIds = $value;

                if (count($productIds) < 1) {
                    return false;
                }

                foreach ($productIds as $product_id) {
                    $childProductId = $product_id;
                    $product = $this->productFactory->setStoreId($storeId)->load($productId);
                    $productType = $product->getTypeId();
                    switch ($productType) {
                        case \Magento\ConfigurableProduct\Model\Product\Type\Configurable::TYPE_CODE:
                            $productOptions = $this->trackingInventoryHelper->getConfigurableProductOptionIds($productId, $childProductId);

                            $productDetail = $this->productFactory->setStoreId($storeId)->load($productId);

                            $params = ['product' => $productId,
                                'selected_configurable_option' =>  $childProductId,
                                'super_attribute' =>  $productOptions,
                                'qty' => $qty];

                            break;
                        case \Magento\GroupedProduct\Model\Product\Type\Grouped::TYPE_CODE:
                            $option = [$childProductId => $qty];
                            $params = ['super_group' =>  $option];

                            $productDetail = $this->productFactory->setStoreId($storeId)->load($productId);
                            break;
                        default:
                            $productDetail = $this->productFactory->setStoreId($storeId)->load($productId);
                            $params = ['qty' => $qty];
                            break;
                    }
                    $this->cart->addProduct($productDetail, $params);
                }

                if ($productType === 'bundle') {
                    $bundleOption = $this->trackingInventoryHelper->getBundledProductOptionIds($productId, $value);

                    foreach ($bundleOption as $key2 => $value2) {
                        $bundleOptionQty[$key2] = $qty;
                    }

                    $productDetail = $this->productFactory->setStoreId($storeId)->load($productId);

                    $params = ['bundle_option' => $bundleOption,
                        'qty' => $qty,
                        'product' => $productId];
                    $this->cart->addProduct($productDetail, $params);
                }
            }
            $this->cart->save();
        } catch (\Exception $e) {
            $this->logger->critical($e);
        }

        return $resultRedirect;
    }
}
