<?php
/*
 * @author TargetBay
 * @copyright - Sathishkumar Mariappan <sathishkumar.m@innoppl.com>
 * @package Targetbay_Tracking
 */

namespace Targetbay\Tracking\Helper;

use Magento\Checkout\Model\Session as CheckoutSession;
use Magento\Customer\Model\Session as CustomerSession;
use Magento\Framework\App\RequestInterface as RequestInterface;
use Magento\Framework\Escaper as EscaperObject;
use Magento\Framework\Registry as RegistryObject;
use Magento\Framework\Session\Generic as CoreSession;

/**
 * Class Data
 *
 * Custom helper class
 *
 * @package Targetbay\Tracking\Helper
 */
class Data extends \Magento\Framework\App\Helper\AbstractHelper
{
    const ANONYMOUS_USER = 'anonymous';
    const ALL_PAGES = 'all';
    const PAGE_VISIT = 'page-visit';
    const PAGE_REFERRAL = 'referrer';

    const ADD_TO_CART = 'add-to-cart';

    const BILLING = 'billing';
    const SHIPPING = 'shipping';
    const TIMEOUT = 900;

    // order fullfillment process
    const ORDER_SHIPMENT = 'shipment';
    const ORDER_INVOICE = 'invoice';
    const ORDER_REFUND = 'creditmemo';

    const CREATE_ACCOUNT = 'create-account';
    const ADMIN_ACTIVATE_ACCOUNT = 'admin-activate-customer-account';
    const LOGIN = 'login';
    const LOGOUT = 'logout';

    // Product stock status.
    const IN_STOCK = 'in-stock';
    const OUT_OF_STOCK = 'out-stock';

    const MODULE_NAME = 'Targetbay_Tracking';
    const RATINGS_STATS = 'ratings-stats';
    const QUESTION_STATS = 'qa-stats';

    const VISITOR_ID = 6000000;

    const STATUS_SUBSCRIBED = 1;
    const STATUS_NOT_ACTIVE = 2;
    const STATUS_UNSUBSCRIBED = 3;
    const STATUS_UNCONFIRMED = 4;

    const HOST_STAGE = 'https://stage.targetbay.com/api/v1/webhooks/';
    const HOST_LIVE = 'https://app.targetbay.com/api/v1/webhooks/';
    const HOST_DEV = 'https://dev.targetbay.com/api/v1/webhooks/';

    const API_STAGE = 'stage';
    const API_LIVE = 'app';
    const API_DEV = 'dev';

    /**
     * @var \Magento\Framework\App\Config\ScopeConfigInterface
     */
    public $scopeConfig;

    /**
     * @var \Magento\Directory\Api\CountryInformationAcquirerInterface
     */
    public $countryInformation;

    /**
     * @var \Magento\Framework\Stdlib\CookieManagerInterface
     */
    public $cookieManager;

    /**
     * @var \Magento\Framework\Stdlib\DateTime\DateTime
     */
    public $date;

    /**
     * @var \Magento\Framework\Url\Helper\Data
     */
    public $urlHelper;

    /**
     * @var \Magento\Framework\HTTP\PhpEnvironment\RemoteAddress
     */
    public $remoteAddress;

    /**
     * @var \Magento\Framework\HTTP\Header
     */
    public $httpHeader;

    /**
     * @var \Magento\Framework\View\Page\Title
     */
    public $pageTitle;

    /**
     * @var CoreSession
     */
    public $coreSession;

    /**
     * @var \Magento\Customer\Model\Session
     */
    public $customerSession;

    /**
     * @var \Magento\Checkout\Model\Session
     */
    public $checkoutSession;

    /**
     * @var \Magento\Store\Model\StoreManagerInterface
     */
    public $storeManager;

    /**
     * @var \Magento\Framework\HTTP\ZendClientFactory
     */
    public $httpClientFactory;

    /**
     * @var \Magento\Framework\UrlInterface
     */
    public $urlInterface;

    /**
     * @var \Magento\Framework\Module\ModuleListInterface
     */
    public $moduleList;

    /**
     * @var \Psr\Log\LoggerInterface
     */
    public $logger;

    /**
     * @var \Magento\Framework\ObjectManagerInterface
     */
    public $objectManager;

    /**
     * @var RequestInterface
     */
    public $request;

    /**
     * @var RegistryObject
     */
    public $registryObj;

    /**
     * @var EscaperObject
     */
    public $escapeObj;

    /**
     * @var \Magento\Framework\App\CacheInterface $cacheType
     */
    public $cacheType;

    /**
     * @var array of modules
     */
    public $moduleNameArray = [
        'catalogsearch',
        'newsletter',
        'checkout',
        'sales',
        'customer',
        'wishlist',
        'review',
        'downloadable',
        'rma',
        'reward',
        'giftcard',
        'storecredit'
    ];

    protected $_cookieMetadataFactory;

    protected $_postdata;

    /**
     * @param \Magento\Framework\Stdlib\DateTime\DateTime $date
     * @param \Magento\Framework\Stdlib\CookieManagerInterface $cookieManager
     * @param \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig
     * @param \Magento\Directory\Api\CountryInformationAcquirerInterface $countryInformation
     * @param \Magento\Framework\Url\Helper\Data $urlHelper
     * @param \Magento\Framework\HTTP\PhpEnvironment\RemoteAddress $remoteAddress
     * @param \Magento\Framework\HTTP\Header $httpHeader
     * @param \Magento\Framework\View\Page\Title $pageTitle
     * @param CoreSession $coreSession
     * @param CustomerSession $customerSession
     * @param CheckoutSession $checkoutSession
     * @param \Magento\Store\Model\StoreManagerInterface $storeManager
     * @param \Magento\Framework\HTTP\ZendClientFactory $httpClientFactory
     * @param \Magento\Framework\UrlInterface $urlInterface
     * @param \Psr\Log\LoggerInterface $logger
     * @param \Magento\Framework\Module\ModuleListInterface $moduleList
     * @param \Magento\Framework\ObjectManagerInterface $objectManager
     * @param RequestInterface $request
     * @param RegistryObject $registryObj
     * @param EscaperObject $escapeObj
     * @param \Magento\Framework\App\CacheInterface $cacheType
     */

    public function __construct(
        \Magento\Framework\Stdlib\DateTime\DateTime $date,
        \Magento\Framework\Stdlib\CookieManagerInterface $cookieManager,
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
        \Magento\Directory\Api\CountryInformationAcquirerInterface $countryInformation,
        \Magento\Framework\Url\Helper\Data $urlHelper,
        \Magento\Framework\HTTP\PhpEnvironment\RemoteAddress $remoteAddress,
        \Magento\Framework\HTTP\Header $httpHeader,
        \Magento\Framework\View\Page\Title $pageTitle,
        CoreSession $coreSession,
        CustomerSession $customerSession,
        CheckoutSession $checkoutSession,
        \Magento\Store\Model\StoreManagerInterface $storeManager,
        \Magento\Framework\HTTP\ZendClientFactory $httpClientFactory,
        \Magento\Framework\UrlInterface $urlInterface,
        \Psr\Log\LoggerInterface $logger,
        \Magento\Framework\Module\ModuleListInterface $moduleList,
        \Magento\Framework\ObjectManagerInterface $objectManager,
        RequestInterface $request,
        RegistryObject $registryObj,
        EscaperObject $escapeObj,
        \Magento\Framework\App\CacheInterface $cacheType,
        \Magento\Framework\Stdlib\Cookie\CookieMetadataFactory $cookieMetadataFactory,
        \Targetbay\Tracking\Helper\Postdata $postdata
    ) {
        $this->_cookieManager = $cookieManager;
        $this->_date = $date;
        $this->_scopeConfig = $scopeConfig;
        $this->_countryInformation = $countryInformation;
        $this->_urlHelper = $urlHelper;
        $this->_remoteAddress = $remoteAddress;
        $this->_httpHeader = $httpHeader;
        $this->_pageTitle = $pageTitle;
        $this->_coreSession = $coreSession;
        $this->_customerSession = $customerSession;
        $this->_checkoutSession = $checkoutSession;
        $this->_storeManager = $storeManager;
        $this->_httpClientFactory = $httpClientFactory;
        $this->_urlInterface = $urlInterface;
        $this->_logger = $logger;
        $this->_moduleList = $moduleList;
        $this->_objectManager = $objectManager;
        $this->_request = $request;
        $this->_registryObj = $registryObj;
        $this->_escapeObj = $escapeObj;
        $this->_cacheType = $cacheType;
        $this->_cookieMetadataFactory = $cookieMetadataFactory;
        $this->_postdata = $postdata;
    }

    /**
     * Check module is enabled or not
     *
     * @return boolean
     */
    public function trackingEnabled()
    {
        return (bool)$this->_scopeConfig->getValue(
            'targetbay_tracking/tracking_groups/enabled',
            \Magento\Store\Model\ScopeInterface::SCOPE_STORE
        );
    }

    /**
     * Get Target bay Host
     *
     * @return mixed
     */
    public function getHostname()
    {
        return $this->_scopeConfig->getValue(
            'targetbay_tracking/tracking_groups/hostname',
            \Magento\Store\Model\ScopeInterface::SCOPE_STORE
        );
    }

    /**
     * Get TargetbayToken
     *
     * @return mixed
     */
    public function getApiToken()
    {
        return $this->_scopeConfig->getValue(
            'targetbay_tracking/tracking_groups/api_token',
            \Magento\Store\Model\ScopeInterface::SCOPE_STORE
        );
    }

    /**
     * Get getApiStatus
     *
     * @return mixed
     */
    public function getApiStatus()
    {
        return $this->_scopeConfig->getValue(
            'targetbay_tracking/tracking_groups/api_status',
            \Magento\Store\Model\ScopeInterface::SCOPE_STORE
        );
    }

    /**
     * Get TargetBayIndex
     *
     * @return mixed
     */
    public function getApiIndex()
    {
        return $this->_scopeConfig->getValue(
            'targetbay_tracking/tracking_groups/api_index',
            \Magento\Store\Model\ScopeInterface::SCOPE_STORE
        );
    }

    /**
     * Enabled the log or not
     *
     * @return string
     */
    public function logEnabled()
    {
        return $this->_scopeConfig->getValue(
            'targetbay_tracking/tracking_groups/debug',
            \Magento\Store\Model\ScopeInterface::SCOPE_STORE
        );
    }

    /**
     * Get the log file name from configurations
     */
    public function getLogFileName()
    {
        return $this->_scopeConfig->getValue(
            'targetbay_tracking/tracking_groups/debug_file',
            \Magento\Store\Model\ScopeInterface::SCOPE_STORE
        );
    }

    /**
     * Get the available pages from configurations
     *
     * @return array
     */
    public function availablePageTypes()
    {
        $types = $this->_scopeConfig->getValue(
            'targetbay_tracking/tracking_groups/page_types',
            \Magento\Store\Model\ScopeInterface::SCOPE_STORE
        );
        //$typesArray = explode(',', $types);

        return explode(',', $types);
    }

    /**
     * Check back orders is enabled or not
     *
     * @return boolean
     */
    public function getBackorderStatus()
    {
        return $this->_scopeConfig->getValue(
            'targetbay_tracking/tracking_groups/manage_stock',
            \Magento\Store\Model\ScopeInterface::SCOPE_STORE
        );
    }

    /**
     * Get the Session Tracking Js
     *
     * @return mixed
     */
    public function getReviewPageSize()
    {
        $reviewSize = $this->_scopeConfig->getValue(
            'targetbay_tracking/tracking_groups/reviews_per_page',
            \Magento\Store\Model\ScopeInterface::SCOPE_STORE
        );

        if ($reviewSize) {
            $reviewSize = $reviewSize;
        } else {
            $reviewSize = 10;
        }

        return $reviewSize;
    }

    /**
     * Get TargetBay richsnippet type
     *
     * @return mixed
     */
    public function getRichsnippetType()
    {
        return $this->_scopeConfig->getValue('targetbay_tracking/tracking_groups/richsnippets_type');
    }

    /**
     * Get TB Instagram Config settings
     * @return mixed
    */
    public function getInstagramSettings()
    {
        return $this->_scopeConfig->getValue('targetbay_tracking/tracking_groups/enable_instagram');
    }

    /**
     * Get TB AdRoll Config settings
     * @return mixed
    */
    public function getAdRollSettings()
    {
        return $this->_scopeConfig->getValue('targetbay_tracking/tracking_groups/enable_adroll');
    }

     /**
     * Get TB AdRoll Advertisable Id
     * @return mixed
    */
    public function getAdRollAdvertisableId()
    {
        return $this->_scopeConfig->getValue('targetbay_tracking/tracking_groups/adroll_adv_id');
    }

    /**
     * Get TB AdRoll Pixel Id 
     * @return mixed
    */
    public function getAdRollPixelId()
    {
        return $this->_scopeConfig->getValue('targetbay_tracking/tracking_groups/adroll_pixel_id');
    }

    /**
     * Customer welcome email and newsletter email enabled or not
     *
     * @return string
     */
    public function getEmailStatus()
    {
        $emailStatus = $this->_scopeConfig->getValue(
            'targetbay_tracking/tracking_groups/disable_email',
            \Magento\Store\Model\ScopeInterface::SCOPE_STORE
        );

        if ($emailStatus) {
            $emailStatus = $emailStatus;
        } else {
            $emailStatus = 1;
        }

        return $emailStatus;
    }

    /**
     * Get the Module version
     *
     * @return mixed
     */
    public function getModuleVersion()
    {
        return $this->_moduleList->getOne(self::MODULE_NAME)['setup_version'];
    }

    /**
     * Get Tracking Code
     *
     * @return mixed
     */
    public function getTrackingScript()
    {
        return $this->_scopeConfig->getValue(
            'targetbay_tracking/tracking_groups/tracking_script',
            \Magento\Store\Model\ScopeInterface::SCOPE_STORE
        );
    }

    /**
     * Check coupon code generation is enabled or not
     *
     * @return boolean
     */
    public function getCouponCodeStatus()
    {
        return $this->_scopeConfig->getValue(
            'targetbay_tracking/coupon_configuration/enabled',
            \Magento\Store\Model\ScopeInterface::SCOPE_STORE
        );
    }

    /**
     * Get coupon code length
     *
     * @return int
     */
    public function getCouponCodeLength()
    {
        return $this->_scopeConfig->getValue(
            'targetbay_tracking/coupon_configuration/length',
            \Magento\Store\Model\ScopeInterface::SCOPE_STORE
        );
    }

    /**
     * Get coupon code format
     *
     * @return string
     */
    public function getCouponCodeFormat()
    {
        return $this->_scopeConfig->getValue(
            'targetbay_tracking/coupon_configuration/format',
            \Magento\Store\Model\ScopeInterface::SCOPE_STORE
        );
    }

    /**
     * Check the page configurations
     *
     * @param $pageType
     * @return boolean
     */
    public function canTrackPages($pageType)
    {
        if (!$this->trackingEnabled()) {
            $this->debug('TargetBay tracking module is not enabled. Please enable it.');
            return false;
        }

        // $availablePages = ['all'];
        $availablePages = $this->availablePageTypes();
        if (in_array(self::ALL_PAGES, $availablePages, true)) {
            return true;
        }

        if (!in_array($pageType, $availablePages, true)) {
            $this->debug("'$pageType'" . 'page is not enabled');
            return false;
        }
        return true;
    }

    /**
     * Basic visit info
     *
     * @return array
     */
    public function visitInfo()
    {
        $customerObj = $this->_objectManager->get('Magento\Customer\Model\Customer');
        $moduleName = $this->_request->getModuleName();
        $data = [];

        try {
            if (in_array($moduleName, $this->moduleNameArray, true)
                && $this->_customerSession->isLoggedIn()) {
                $user_id = $this->_customerSession->getCustomerId();
            } else {
                $user_id = $this->_cookieManager->getCookie('targetbay_session_id');
            }

            if ($this->_customerSession->isLoggedIn()) {
                $customerData = $customerObj->load($this->_customerSession->getCustomerId());
                $data['user_name'] = $customerData->getFirstname() . ' ' . $customerData->getLastname();
                $data['session_id'] = $this->_coreSession->getCustomerSessionId();
            } else {
                $data['user_name'] = self::ANONYMOUS_USER;
                $data['session_id'] = $this->_cookieManager->getCookie('targetbay_session_id');
            }

            $data['user_id'] = $user_id;

            $data['page_url'] = $this->_urlInterface->getCurrentUrl();
            $data['ip_address'] = $this->_remoteAddress->getRemoteAddress();
            $data['user_agent'] = $this->_httpHeader->getHttpUserAgent();

            if ($this->_cookieManager->getCookie('utm_source')) {
                $data['utm_sources'] = $this->_cookieManager->getCookie('utm_source');
            } else {
                $data['utm_sources'] = '';
            }

            if ($this->_cookieManager->getCookie('utm_token')) {
                $data['utm_token'] = $this->_cookieManager->getCookie('utm_token');
            } else {
                $data['utm_token'] = '';
            }

            if ($this->_pageTitle->getShort()) {
                $pageTitle = $this->_pageTitle->getShort();
            } else {
                $pageTitle = $this->_checkoutSession->getTitle();
            }

            $data['page_title'] = $this->_escapeObj->escapeQuote($pageTitle);
        } catch (\Exception $e) {
            $this->_logger->critical($e);
        }

        return $data;
    }

    /**
     * Push the referrer url
     *
     * @return array|bool
     */
    public function getRefererData()
    {
        $domainName = $_SERVER['SERVER_NAME'];
        $referrer = $this->_httpHeader->getHttpReferer();

        if ($referrer == '' || strpos($referrer, $domainName) !== false) {
            return false; // base url and referrer url matches.
        }

        $data = $this->visitInfo();
        $data['referrer_url'] = $referrer;

        return $data;
    }

    /**
     * Stop the page visit for already tracked events
     *
     * @param $eventName
     *
     * @return bool
     */
    public function eventAlreadyTracked($eventName)
    {
        $stopAction = [
            'cms_index_index',
            'catalog_category_view',
            'catalog_product_view',
            'catalogsearch_result_index',
            'catalogsearch_ajax_suggest',
            'checkout_cart_add',
            'checkout_cart_index',
            'checkout_cart_updatePost',
            'checkout_cart_remove',
            'customer_account_create',
            'customer_account_loginPost',
            'customer_account_logout',
            'customer_account_logoutSuccess',
            'customer_account_createPost',
            'checkout_onepage_index',
            'checkout_onepage_saveBilling',
            'checkout_onepage_saveShipping',
            'checkout_onepage_saveShippingMethod',
            'checkout_onepage_savePayment',
            'checkout_onepage_getAdditional',
            'checkout_onepage_progress',
            'checkout_onepage_saveOrder'
        ];

        return in_array($eventName, $stopAction, true);
    }

    /**
     * Push the page info using Varien_Http_Client
     *
     * @param $url
     * @param $jsonData
     *
     * @return string
     */
    public function postPageInfo($url, $jsonData)
    {
       return  $this->_postdata->postPageInfo($url, $jsonData);
    }

    /**
     * Get the common info for quote or order ,billing and shipping
     *
     * @param  $object
     * @return string
     */
    public function getSessionInfo()
    {
        if ($this->_customerSession->isLoggedIn()) {
            $data['session_id'] = $this->_customerSession->getCustomer()->getId();
            $data['user_id'] = $this->_customerSession->getCustomer()->getId();
        } else {
            $data['session_id'] = $this->_cookieManager->getCookie('targetbay_session_id');
            $data['user_id'] = $this->_cookieManager->getCookie('targetbay_session_id');
        }

        if ($this->_cookieManager->getCookie('utm_source')) {
            $data['utm_sources'] = $this->_cookieManager->getCookie('utm_source');
        } else {
            $data['utm_sources'] = '';
        }

        if ($this->_cookieManager->getCookie('utm_token')) {
            $data['utm_token'] = $this->_cookieManager->getCookie('utm_token');
        } else {
            $data['utm_token'] = '';
        }

        $data['user_agent'] = $this->_httpHeader->getHttpUserAgent();
        $data['timestamp'] = strtotime($this->_date->date('Y-m-d'));

        if ($this->_pageTitle->getShort()) {
            $pageTitle = $this->_pageTitle->getShort();
        } else {
            $pageTitle = $this->_checkoutSession->getTitle();
        }
        $data['page_title'] = $this->_escapeObj->escapeQuote($pageTitle);
        return $data;
    }

    /**
     * Get the cart info
     *
     * @return array
     */
    public function getCartInfo()
    {
        if ($this->_customerSession->isLoggedIn()) {
            $data['session_id'] = $this->_customerSession->getId();
            $data['user_name'] = $this->_customerSession->getCustomer()->getName();
            $data['user_mail'] = $this->_customerSession->getCustomer()->getEmail();
            $data['user_id'] = $this->_customerSession->getId();
        } else {
            $data['session_id'] = $this->_cookieManager->getCookie('targetbay_session_id');
            $data['user_name'] = self::ANONYMOUS_USER;
            $data['user_mail'] = self::ANONYMOUS_USER;
            $data['user_id'] = $this->_cookieManager->getCookie('targetbay_session_id');
        }

        $data['user_agent'] = $this->_httpHeader->getHttpUserAgent();
        $data['order_id'] = $this->_checkoutSession->getQuoteId();
        $data['timestamp'] = strtotime($this->_date->date('Y-m-d'));

        return $data;
    }

    /**
     * Get the item info
     *
     * @param $item
     * @param $actionType
     *
     * @return unknown
     */
    public function getItemInfo($item, $actionType = false, $productQty = null)
    {
        $dataItem = [];
        try {
            if ($item->getProductId()) {
                $product = $this->_objectManager->get('\Magento\Catalog\Model\Product')
                        ->load($item->getProductId());
                $dataItem['type'] = $item->getProductType();
                $dataItem['product_id'] = $item->getProductId();
                $dataItem['product_name'] = $this->_escapeObj->escapeQuote($item->getName());
                $dataItem['product_sku'] = $product->getSku();
                $dataItem['msrp_price'] = $product->getMsrp() ? $product->getMsrp() : '';
                $dataItem['price'] = $actionType ? $item->getProduct()->getPrice() : $item->getPrice();
                //$qty = $actionType ? $this->_request->getParam('qty') : $item->getQty();

                /* $this->_request->getParam('qty') it's return null value in product list page addto cart.
                   So now we take the product quantity from observer .
                */
                $qty = $actionType ? $productQty : $item->getQty();
                if ($qty == null && $item->getQty()) {
                    $qty = $item->getQty();
                }
                $dataItem['productimg'] = $this->getImageUrl($product, 'image');
                $dataItem['abstract'] = $product->getAbstract();

                $dataItem['category'] = $this->getProductCategory($product);
                $dataItem['category_name'] = $this->getProductCategoryName($product);
                $dataItem['quantity'] = ($item->getData('qty_ordered')) ? $item->getData('qty_ordered') : $qty;
                $dataItem['page_url'] = $product->getUrlModel()->getUrl($product);
            }
        } catch (\Exception $e) {
            $this->_logger->critical($e);
        }

        return $dataItem;
    }

    /**
     * Get the Product image url
     *
     * @param $product
     * @param $imageType
     *
     * @return string
     */
    public function getImageUrl($product, $imageType)
    {
        $image = $product->getData($imageType);
        $imgPath = '';
        if (!empty($image) || $image !== 'no_selection') {
            $imgPath = $this->_storeManager->getStore()
                ->getBaseUrl(\Magento\Framework\UrlInterface::URL_TYPE_MEDIA);
            $imgPath .= 'catalog/product' . $product->getData($imageType);
        }

        return $imgPath;
    }

    /**
     * Get the product categories
     *
     * @param $product
     *
     * @return unknown
     */
    public function getProductCategory($product)
    {
        $categoryIds = $product->getCategoryIds();

        $categoryId = '';
        if (count($categoryIds)) {
            $categoryId = implode(',', $categoryIds);
        }

        return $categoryId;
    }

    /**
     * Get the product categories
     *
     * @param $product
     *
     * @return unknown
     */
    public function getProductCategoryName($product)
    {
        $categoryIds = $product->getCategoryIds();

        $productCategories = [];
        $categoryName = '';
        if (count($categoryIds)) {
            foreach ($categoryIds as $categoryId) {
                $_category = $this->_objectManager->get('Magento\Catalog\Model\Category')
                    ->load($categoryId);
                $productCategories[] = str_replace("'", "", $_category->getName());
            }

            $categoryName = implode(',', $productCategories);
        }

        return $this->_escapeObj->escapeQuote($categoryName);
    }

    /**
     * Get the order or quote address by address type.
     *
     * @param $object
     * @param $type
     *
     * @return string
     */
    public function getAddressData($object, $type)
    {
        if ($type === self::SHIPPING) {
            $address = $object->getShippingAddress();
        } else {
            $address = $object->getBillingAddress();
        }

        $addressData = $this->getSessionInfo();
        $addressData['first_name'] = $address->getFirstname();
        $addressData['last_name'] = $address->getLastname();
        $guestUsername = $address->getFirstname() . ' ' . $address->getLastname();
        $customerName = $addressData['first_name'] . ' ' . $addressData['last_name'];
        $gName = !empty($guestUsername) ? $guestUsername : self::ANONYMOUS_USER;
        $addressData['user_name'] = $object->getCustomerIsGuest() ? $gName : $customerName;
        $addressData['order_id'] = $object->getId();
        $addressData['user_mail'] = $object->getCustomerEmail();
        $addressData['address1'] = $address->getStreet(1);
        $addressData['address2'] = $address->getStreet(2);
        $addressData['city'] = $address->getCity();
        $addressData['state'] = $address->getRegion();
        $addressData['zipcode'] = $address->getPostcode();

        $countryName = '';
        if ($address->getCountryId()) {
            $countryName = $this->_countryInformation->getCountryInfo($address->getCountryId())
                ->getFullNameLocale();
        }

        $addressData['country'] = $countryName !== '' ? $countryName : $address->getCountryId();
        $addressData['phone'] = $address->getTelephone();

        return $addressData;
    }

    /**
     * Collect the order items info
     *
     * @param $order
     * @param $orderExportApi
     *
     * @return array
     */
    public function getOrderItemsInfo($order, $orderExportApi = false)
    {
        try {
            $items = $order->getAllVisibleItems();
            $dataItems = [];
            foreach ($items as $orderItem) {
                // print_r($orderItem->getData()); exit();
                $productCheck = $this->_objectManager->get('\Magento\Catalog\Model\Product');
                // echo $orderItem->getData('sku'); exit;
                if($productCheck->getIdBySku($orderItem->getData('sku'))) {
                    $product = $this->_objectManager->get('\Magento\Catalog\Model\Product')->load($orderItem->getData('product_id'));
                    $productVisibility = $product->getVisibility();
                    if ($productVisibility != 1) {

                        $dataItem = $this->getItemInfo($orderItem);
                        if ($customOptions = $this->getCustomOptionsInfo($orderItem, $orderExportApi)) {
                            $dataItem['attributes'] = $customOptions;
                        }
                        
                        $dataItems[] = $dataItem;
                    }
                }
            }
        } catch (\Exception $e) {
            $this->debug(__($e->getMessage()));
        }
        return $dataItems;
    }

    /**
     * Get the item custom options
     *
     * @param  $item
     * @param  $orderExportApi
     *
     * @return array|bool
     */
    public function getCustomOptionsInfo($item, $orderExportApi)
    {
        if ($item->getProduct() !== null) {
            $productOption = $item->getProduct()->getTypeInstance(true)
                ->getOrderOptions($item->getProduct());
            $customOptions = $orderExportApi ? $item->getProductOptions() : $productOption;

            if (empty($customOptions['options']) && empty($customOptions ['attributes_info'])) {
                return false;
            }

            if (isset($customOptions['attributes_info'])) {
                $superAttributeInfo = $this->getOptionValues($customOptions['attributes_info']);
            } else {
                $superAttributeInfo = [];
            }
            if (isset($customOptions['options'])) {
                $customOptionInfo = $this->getOptionValues($customOptions['options']);
            } else {
                $customOptionInfo = [];
            }

            return array_merge($superAttributeInfo, $customOptionInfo);
        }
    }

    /**
     * Get the options values
     *
     * @param $options
     *
     * @return array
     */
    public function getOptionValues($options)
    {
        $optionData = [];
        $data = [];
        foreach ($options as $option) {
            $data['label'] = $option ['label'];
            $data['value'] = $option ['value'];
            $optionData [] = $data;
        }

        return $optionData;
    }

    /**
     * debugging
     *
     * @param $mess
     */
    public function debug($mess)
    {
        $fileName = $this->getLogFileName();
        if ($this->logEnabled()) {
            $writer = new \Zend\Log\Writer\Stream(BP . '/var/log/' . $fileName);
            $logger = new \Zend\Log\Logger();
            $logger->addWriter($writer);
            $logger->info($mess);
        }
    }

    /**
     * Check the order full fillment
     *
     * @return boolean
     */
    public function isFullFillmentProcess($params)
    {
        if (isset($params[self::ORDER_SHIPMENT]) || isset($params[self::ORDER_INVOICE])
            || isset($params[self::ORDER_REFUND])) {
            return true;
        }
        return false;
    }

    /**
     *
     * @param $order
     * @param $params
     *
     * @return array|boolean
     */
    public function getFullFillmentData($order, $params)
    {
        $shipmentItems = [];
        $shipmentsInfo = [];
        if (isset($params[self::ORDER_SHIPMENT])) {
            try {
                $shipmentsInfo['order_id'] = $order->getId();
                $shipmentsInfo['order_status'] = $order->getStatus();
                $shipmentsInfo['total_ordered_qty'] = $order->getData('total_qty_ordered');
                $guestUser = $order->getData('customer_is_guest');

                if (isset($guestUser) && $guestUser == 1) {
                    $shipmentsInfo['user_id'] = self::ANONYMOUS_USER;
                } else {
                    $shipmentsInfo['user_id'] = $order->getData('customer_id');
                }

                $shipmentsInfo['user_mail'] =  $order->getData('customer_email');
                $shipmentsInfo['created_at'] = $order->getData('updated_at');
                foreach ($order->getAllVisibleItems() as $item) {
                    if ($item->getQtyShipped() === '') {
                        continue;
                    }
                    $shipmentItemInfo['product_id'] = $item->getProductId();
                    $shipmentItemInfo['name'] = $item->getName();
                    $shipmentItemInfo['sku'] = $item->getSku();
                    $shipmentItemInfo['qty_ordered'] = $item->getQtyOrdered();
                    $shipmentItemInfo['qty_shipped'] = $item->getQtyShipped();
                    $shipmentItems[] = $shipmentItemInfo;
                }
                $shipmentsInfo['shipment_items'] = $shipmentItems;

                return $shipmentsInfo;
            } catch (\Exception $e) {
                $this->_logger->critical($e);
            }
        }

        return false;
    }

    /**
     * Check the order placed by registered user or not
     *
     * @param $order
     *
     * @return boolean|Mage_Core_Model_Abstract
     */
    public function isRegisterCheckout($order)
    {
        $checkoutMethod = $this->_objectManager->create('Magento\Quote\Model\Quote')
            ->load($order->getQuoteId())->getCheckoutMethod(true);

        if ($checkoutMethod !== 'register') {
            return false;
        }

        return $this->_objectManager->create('Magento\Customer\Model\Customer')
            ->load($order->getCustomerId());
    }

    /**
     * Get the customer data based on the action
     *
     * @param unknown $customer
     * @param unknown $action
     * @return array
     */
    public function getCustomerData($customer, $action)
    {
        $data = [];
        try {
            switch ($action) {
                case self::LOGIN:
                    $data = $this->getCustomerSessionId($customer);
                    $data['login_date'] = $this->_date->date('Y-m-d');
                    break;
                case self::LOGOUT:
                    $data = [];
                    $data['session_id'] = $this->_coreSession->getCustomerSessionId();
                    $data['logout_date'] = $this->_date->date('Y-m-d');
                    $this->_coreSession->unsTrackingSessionId();
                    $this->_coreSession->unsCustomerSessionId();
                    break;
                case self::CREATE_ACCOUNT:
                    $data = $this->getCustomerSessionId($customer);
                    $data['firstname'] = $customer->getFirstname();
                    $data['lastname'] = $customer->getLastname();
                    $customerGroupId  = $customer->getGroupId();
                    $data['customer_group_id'] = $this->getCustomerGroup($customerGroupId);
                    //$data['subscription_status'] = $this->getSubscriptionStatus($customer->getId());
                    $data['account_created'] = $this->_date->date('Y-m-d');
                    break;
            }           
            $data['user_id'] = $customer->getId();
            $data['user_name'] = $customer->getFirstname() . ' ' . $customer->getLastname();
            $data['user_mail'] = $customer->getEmail();
            $data['timestamp'] = strtotime($this->_date->date('Y-m-d'));
            $data['ip_address'] = $this->_remoteAddress->getRemoteAddress();
        } catch (\Exception $e) {
            $this->_logger->critical($e);
        }

        return $data;
    }

    /**
     * Get the customer address data based on the action
     *
     * @param $addressId
     * @return array
     */
    public function getCustomerAddressData($addressId)
    {
        $data = [];
        try {
            $address = $this->_objectManager->create('Magento\Customer\Model\Address');
            $customer = $this->_customerSession->getCustomer();
            $data['user_id'] = $customer->getId();
            $data['user_name'] = $customer->getName();
            $data['user_mail'] = $customer->getEmail();
            if ($addressId) {
                $existsAddress = $address->load($addressId);
                $data['address_id'] = $addressId;
                $data['old_address'] = $this->getExistingAddressData($existsAddress);
            }

            $data['new_address'] = $this->getNewAddressData();
            $data['timestamp'] = strtotime($this->_date->date('Y-m-d'));
            $data['ip_address'] = $this->_remoteAddress->getRemoteAddress();
            $data['user_agent'] = $this->_httpHeader->getHttpUserAgent();
        } catch (\Exception $e) {
            $this->_logger->critical($e);
        }

        return $data;
    }

    /**
     * Get the customer address data based on the action
     *
     * @return array
     */
    public function getNewAddressData()
    {
        //$request = Mage::app()->getRequest();
        $addressData = [];

        try {
            $addressData['firstname'] = $this->_request->getParam('firstname');
            $addressData['lastname'] = $this->_request->getParam('lastname');
            $addressData['company'] = $this->_request->getParam('company');
            $addressData['street'] = $this->_request->getParam('street');
            $addressData['city'] = $this->_request->getParam('city');
            $addressData['region_id'] = $this->_request->getParam('region_id');
            $addressData['region'] = $this->_request->getParam('region');
            $addressData['postcode'] = $this->_request->getParam('postcode');
            $addressData['country_id'] = $this->_request->getParam('country_id');
            $addressData['telephone'] = $this->_request->getParam('telephone');
        } catch (\Exception $e) {
            $this->_logger->critical($e);
        }

        return $addressData;
    }

    /**
     * Get the customer existing address data.
     *
     * @param $existsAddress
     *
     * @return array
     */
    public function getExistingAddressData($existsAddress)
    {
        $addressData = [];
        try {
            $addressData['firstname'] = $existsAddress->getFirstname();
            $addressData['lastname'] = $existsAddress->getLastname();
            $addressData['company'] = $existsAddress->getCompany();
            $addressData['street'] = $existsAddress->getStreet();
            $addressData['city'] = $existsAddress->getCity();
            $addressData['region_id'] = $existsAddress->getRegionId();
            $addressData['region'] = $existsAddress->getRegion();
            $addressData['postcode'] = $existsAddress->getPostcode();
            $addressData['country_id'] = $existsAddress->getCountryId();
            $addressData['telephone'] = $existsAddress->getTelephone();
        } catch (\Exception $e) {
            $this->_logger->critical($e);
        }

        return $addressData;
    }

    /**
     * Get the customer subscription status
     *
     * @param $customerId
     *
     * @return string
     */
    public function getSubscriptionStatus($customerId)
    {
        try {
            $subscriberFactory = $this->_objectManager->create('\Magento\Newsletter\Model\SubscriberFactory')
                ->loadByCustomerId($customerId);
            $status = '';

            if (!empty($subscriberFactory)) {
                switch ($subscriberFactory->getSubscriberStatus()) {
                    case self::STATUS_UNSUBSCRIBED:
                        $status = 'Unsubscribed';
                        break;
                    case self::STATUS_SUBSCRIBED:
                        $status = 'Subscribed';
                        break;
                    case self::STATUS_UNCONFIRMED:
                        $status = 'Unconfirmed';
                        break;
                    case self::STATUS_NOT_ACTIVE:
                        $status = 'Not Activated';
                        break;
                    default:
                        $status = $subscriberFactory->getSubscriberStatus();
                        break;
                }
            }

            return $status;
        } catch (\Exception $e) {
            $this->_logger->critical($e);
        }
    }

    /**
     * Get the customer session info
     *
     * @param $customer
     *
     * @return array
     */
    public function getCustomerSessionId($customer)
    {
        $data = [];

        $visitor_data = $this->_coreSession->getVisitorData();
        $visitorSessionId = $visitor_data['session_id'];
        $currentDate = date('Y-m-d H:i:s');
        $sessionId = $visitorSessionId ? $visitorSessionId : $visitorSessionId . strtotime($currentDate);
        $data['session_id'] = $sessionId;
        $data['previous_session_id'] = $this->_cookieManager->getCookie('targetbay_session_id');
        $this->_coreSession->setCustomerSessionId($sessionId);
        return $data;
    }

    /**
     * Remove the cookies
     */
    public function removeCookies()
    {
        $metadata = $this->_cookieMetadataFactory->createPublicCookieMetadata()
                        ->setDuration(self::TIMEOUT)
                        ->setPath('/')
                        ->setDomain($this->_coreSession->getCookieDomain())
                        ->setHttpOnly(false);

        $this->_cookieManager->deleteCookie('trackingsession', $metadata);
        $this->_cookieManager->deleteCookie('targetbay_session_id', $metadata);
        $this->_cookieManager->deleteCookie('trackingid', $metadata);
        $this->_cookieManager->deleteCookie('trackingemail', $metadata);
        $this->_cookieManager->deleteCookie('trackingname', $metadata);
        $this->_cookieManager->deleteCookie('trackingorderid', $metadata);
        $this->_cookieManager->deleteCookie('utm_source', $metadata);
        $this->_cookieManager->deleteCookie('utm_token', $metadata);
        $this->_cookieManager->deleteCookie('user_loggedin', $metadata);
        $this->_cookieManager->deleteCookie('tb_fetch_points', $metadata);

    }

    /**
     * Get the Order data
     *
     * @param $object
     *
     * @return array
     */
    public function getInfo($object)
    {
        $data = $this->getSessionInfo();

        if ($object->getCustomerIsGuest()) {
            $billingAddress = $object->getBillingAddress();
            $data['first_name'] = $billingAddress->getFirstname();
            $data['last_name'] = $billingAddress->getLastname();
            $guestUsername = $data['first_name'] . ' ' . $data['last_name'];
        } else {
            $data['first_name'] = $object->getCustomerFirstname();
            $data['last_name'] = $object->getCustomerLastname();
            $guestUsername = '';
        }

        $gName = !empty($guestUsername) ? $guestUsername : self::ANONYMOUS_USER;
        $customerName = $data['first_name'] . ' ' . $data['last_name'];
        $data['user_name'] = $object->getCustomerIsGuest() ? $gName : $customerName;
        $data['user_mail'] = $object->getCustomerEmail();
        $data['order_price'] = $object->getSubtotal();
        $data['order_quantity'] = $object->getData('total_qty_ordered');
        $data['shipping_method'] = $object->getData('shipping_description');
        $data['shipping_price'] = $object->getData('shipping_amount');
        $data['coupon_code'] = $object->getData('coupon_code');
        $data['discount_amount'] = $object->getData('discount_amount');
        $data['tax_amount'] = $object->getData('tax_amount');
        $data['payment_title'] = $object->getPayment()->getMethodInstance()->getTitle();

        return $data;
    }
    /**
     * Get the product info
     *
     * @param $product
     *
     * @return array
     */
    public function getProductData($product)
    {
        $data = [];
        $data['image_url'][] = $this->getProductImages($product);
        $data['entity_id'] = $product->getId();
        $data['attribute_set_id'] = $product->getEntityTypeId();
        $data['type_id'] = $product->getTypeId();
        $data['sku'] = $product->getSku();
        $data['product_status'] = $product->getStatus();
        $data['currency_type'] = $this->_storeManager->getStore()->getCurrentCurrency()->getCode();
        $data['stock_count'] = -1;

        if ($stock = $product->getData('stock_data')) {
            $data['stock_count'] = !empty($stock ['is_in_stock']) ? $stock ['qty'] : -1;
        }

        $data['visibility'] = $product->getVisibility();
        $data['description'] = $product->getDescription();
        $data['msrp_price'] = $product->getMsrp();
        $data['price'] = $product->getPrice();
        $data['weight'] = $product->getWeight();
        $data['name'] = $product->getName();

        $data['category'] = $this->getProductCategory($product);

        // Get product url key.
        if (!empty($product->getUrlKey())) {
            $urlKey = $product->getUrlKey();
        } else {
            $urlKey = $product->getProductUrl();
        }
        $data['url_key'] = $product->getUrlKey();
        $data['full_url_key'] = $product->getProductUrl();

        $configOptions = [];
        $customOptions = [];

        if ($configData = $product->getData('configurable_attributes_data')) {
            $configOptions = $this->productOptions($configData, 'label');
        }

        if ($custOptions = $product->getData('product_options')) {
            $customOptions = $this->productOptions($custOptions);
        }

        $options = array_merge($configOptions, $customOptions);
        if (!empty($options)) {
            $data['attributes'] = $options;
        }

        return $data;
    }

    /**
     * Get the product options when saving product
     *
     * @param $configData
     * @param string $customOption
     * @return array
     */
    public function productOptions($configData, $customOption = 'title')
    {
        $options = [];
        foreach ($configData as $cdata) {
            $attrLabel = $cdata [$customOption];
            if (!isset($cdata ['values'])) {
                $options [] = [
                    'label' => $attrLabel,
                    'value' => $attrLabel
                ];
                continue;
            }
            foreach ($cdata['values'] as $val) {
                if (!isset($cdata [$customOption])) {
                    $attrVal = $val [$customOption];
                    $options [] = [
                        'label' => $attrLabel,
                        'value' => $attrVal
                    ];
                }
            }
        }

        return $options;
    }

    /**
     * Get the product images
     *
     * @param $product
     *
     * @return array
     */
    public function getProductImages($product)
    {
        $images = [];
        $images ['url'] = $this->getImageUrl($product, 'image');
        $images ['position'] = 1;
        $images ['thumbnail_image'] = $this->getImageUrl($product, 'small_image');
        $images ['medium_image'] = $this->getImageUrl($product, 'thumbnail');

        return $images;
    }

    /**
     * Get the wishlist product info
     *
     * @param $productId
     *
     * @return array
     */
    public function getWishlistProductInfo($productId)
    {
        $product = $this->_objectManager->get('Magento\Catalog\Model\Product')->load($productId);

        $data = [];
        $qty = 1;

        $data['type'] = $product->getTypeId();
        $data['product_id'] = $product->getId();
        $data['product_sku'] = $product->getSku();
        $data['name'] = $this->_escapeObj->escapeQuote($product->getName());
        $data['msrp_price'] = $product->getMsrp();
        $data['price'] = $product->getPriceInfo()->getPrice('final_price')->getAmount()
            ->getBaseAmount();
        $data['special_price'] = $product->getSpecialPrice();
        $data['productimg'] = $this->getImageUrl($product, 'image');
        $data['category'] = $this->getProductCategory($product);
        $data['abstract'] = $product->getAbstract();
        $data['category_name'] = $this->getProductCategoryName($product);
        if ($this->_request->getParam('qty')) {
            $data['quantity'] = $this->_request->getParam('qty');
        } else {
            $data['quantity'] = $qty;
        }
        $data['page_url'] = $product->getUrlModel()->getUrl($product);

        return $data;
    }

    /**
     * Get the quote item info
     *
     * @param $items
     * @param $orderExportApi
     *
     * @return array
     */
    public function getQuoteItems($items, $orderExportApi = false)
    {
        $dataItems = [];
        foreach ($items as $orderItem) {
            $dataItem = $this->getItemInfo($orderItem);
            if ($customOptions = $this->getCustomOptionsInfo($orderItem, $orderExportApi)) {
                $dataItem['attributes'] = $customOptions;
            }
            $dataItems[] = $dataItem;
        }

        return $dataItems;
    }

    /**
     * Collect the wishlist items info
     *
     * @param $wishlistInfo
     *
     * @return array
     */
    public function getWishlistItemsInfo($wishlistInfo)
    {
        $dataItems = [];
        $wishlistItemCollection = $wishlistInfo->setStoreId(1)->getItemCollection();

        foreach ($wishlistItemCollection as $id => $wishlistItem) {
            $product = $wishlistItem->getProduct();
            $dataItem = $this->wishlistProductInfo($product);
            $dataItems[$id] = $dataItem;
        }

        return $dataItems;
    }

    /**
     * Collect the wishlist items product info
     *
     * @param $product
     *
     * @return array
     */
    public function wishlistProductInfo($product)
    {
        $product = $this->_objectManager->get('Magento\Catalog\Model\Product')
            ->load($product->getData('entity_id'));
        $dataItem = [];
        $dataItem['type'] = $product->getTypeId();
        $dataItem['product_id'] = $product->getId();
        $dataItem['product_name'] = $this->_escapeObj->escapeQuote($product->getName());
        $dataItem['msrp_price'] = $product->getMsrp();
        $dataItem['price'] = $product->getPrice();
        $dataItem['productimg'] = $this->getImageUrl($product, 'image');
        $dataItem['abstract'] = $product->getAbstract();
        $dataItem['category'] = $this->getProductCategory($product);
        $dataItem['category_name'] = $this->getProductCategoryName($product);
        $dataItem['page_url'] = $product->getUrlModel()->getUrl($product);

        return $dataItem;
    }

    /**
     * Get API url for static pages
     *
     * @return string
     */
    public function getApiUrl()
    {
        $controllerName = $this->_request->getControllerName();
        $moduleName = $this->_request->getModuleName();
        $type = '';

        if ($moduleName === 'cms' || $moduleName === 'brand') {
            $type = 'page-visit';
            $endPointUrl = $this->getHostname() . $type . '?api_token=' . $this->getApiToken();
        } elseif ($controllerName === 'category') {
            $type = 'category-view';
            $endPointUrl = $this->getHostname() . $type . '?api_token=' . $this->getApiToken();
        } elseif ($controllerName === 'product') {
            $type = 'product-view';
            $endPointUrl = $this->getHostname() . $type . '?api_token=' . $this->getApiToken();
        }

        return $type;
    }

    /**
     * Get page visit info for static page
     *
     * @return array
     */
    public function getPageInfo()
    {
        $controllerName = $this->_request->getControllerName();
        $moduleName = $this->_request->getModuleName();

        if ($moduleName === 'cms' || $moduleName === 'brand') {
            $data = $this->getPageVisitData();
        } elseif ($controllerName === 'category') {
            $data = $this->getCategoryViewData();
        } elseif ($controllerName === 'product') {
            $data = $this->getProductViewData();
        }
        $data['index_name'] = $this->getApiIndex();

        return $data;
    }

    /**
     * Visiting page info
     *
     * @return array
     */
    public function getPageVisitData()
    {
        // $eventHandle = $this->_objectManager->get('Targetbay\Tracking\Observer\EventHandles');
        // $eventHandle->setCookieValues();

        $this->setCookieValuesWithEncode();

        return $this->visitInfo();
    }

    /**
     * Category view page
     *
     * @return array
     */
    public function getCategoryViewData()
    {
        try{
            $category = $this->_registryObj->registry('current_category');
            if(empty($category)){
                $cId = $this->_request->getParam('id');
                $category = $this->_objectManager->get('Magento\Catalog\Model\Category')->load($cId);
            }
            if (!empty($category)) {
                $data = $this->visitInfo();
                $data['category_id'] = $category->getId();
                $data['category_url'] = $category->getUrl();
                $data['category_name'] = $this->_escapeObj->escapeQuote($category->getName());

                return $data;
            }
        } catch (\Exception $e) {
            $this->debug('Error :' . $e);
        }
    }

    /**
     * Product view page
     *
     * @return array
     */
    public function getProductViewData()
    {
        $data = $this->visitInfo();
        $productData = $this->_registryObj->registry('product');
	 if($productData){
	    $pId = $productData->getId();
        }else{
            $pId = $this->_request->getParam('id');
        }
        $product = $this->_objectManager->get('Magento\Catalog\Model\Product')
            ->load($pId);
        $categoryIds = $product->getCategoryIds();

        if (count($categoryIds)) {
            $firstCategoryId = $categoryIds[0];
            $_category = $this->_objectManager->get('Magento\Catalog\Model\Category')
                ->load($firstCategoryId);
            $data['category'] = $this->_escapeObj->escapeQuote($_category->getName());
        }
        $data['product_id'] = $product->getId();
        $data['product_name'] = $this->_escapeObj->escapeQuote($product->getName());
        $data['msrp_price'] = $product->getMsrp();
        $data['price'] = $product->getPrice();
        $data['productimg'] = $this->getImageUrl($product, 'image');
        $data['abstract'] = $product->getAbstract();

        $data['stock'] = self::OUT_OF_STOCK;
        if ($product->isAvailable()) {
            $data['stock'] = self::IN_STOCK;
        }

        return $data;
    }

    /**
     * Get the TargetBay review count and rating for product
     *
     * @return array
     */
    public function getRichSnippets()
    {
        if (!$this->trackingEnabled()) {
            return false;
        }

        try {
            $productData = $this->_registryObj->registry('product');
            $productId = $productData->getId();
            $cacheResponseId = 'product_tbreview_response_' . $productId;
            $cacheTbCountId = 'product_tbreview_count_' . $productId;
            $cacheTbQaCountId = 'qa_tbreview_count_' . $productId;
            //$responseBody = '';
            //$responseData = array();
            $type = self::RATINGS_STATS;
            $cacheKey = 'block_html';

            if (false !== ($responseBody = $this->_cacheType->load($cacheResponseId))) {
                $responseBody = unserialize($responseBody);
            }

            if (false !== ($cacheTbReviewCount = $this->_cacheType->load($cacheTbCountId))) {
                $cacheTbReviewCount = unserialize($cacheTbReviewCount);
            }

            if (false !== ($cacheTbQaCount = $this->_cacheType->load($cacheTbQaCountId))) {
                $cacheTbQaCount = unserialize($cacheTbQaCount);
            }

            if (!empty($responseBody) && isset($responseBody->tbReviewCount) &&
                ($responseBody->tbReviewCount !== '') &&
                (($responseBody->tbReviewCount == $cacheTbReviewCount) &&
                    ($responseBody->tbQACount == $cacheTbQaCount))) {
                $responseData = $responseBody->tbContent;

                return $responseData;
            } else {
                $data['product_id'] = $productId;
                $data['index_name'] = $this->getApiIndex();
                $data['qaDisplay'] = 1;
                $jsonData = json_encode($data);
                $apiToken = '?api_token=' . $this->getApiToken();
                $feedUrl = $this->getHostname() . $type . $apiToken;
                // $this->_logger->info('Url===>' . $feedUrl);
                // $this->_logger->info('jsonData===>');
                // $this->_logger->info(print_r($jsonData, true));
                $response = $this->postPageInfo($feedUrl, $jsonData);

                $responseBody = json_decode($response);
                if (!empty($responseBody)) {
                    $this->_cacheType->remove($cacheResponseId);
                    $this->_cacheType->remove($cacheTbCountId);
                    $this->_cacheType->remove($cacheTbQaCountId);
                    $this->_cacheType->save(serialize($responseBody), $cacheResponseId, [$cacheKey]);
                    $this->_cacheType->save(serialize($responseBody->tbReviewCount), $cacheTbCountId, [$cacheKey]);
                    $this->_cacheType->save(serialize($responseBody->tbQACount), $cacheTbQaCountId, [$cacheKey]);
                    $responseData = $responseBody->tbContent;

                    return $responseData;
                }
            }
        } catch (\Exception $e) {
            $this->debug('Error :' . $e);
        }
        return [];
    }

    /**
     * Get the TargetBay review count and dynamic ids
     *
     * @return array
     */
    public function getTargetbayReviewId()
    {
        try {
            $trackingSnippet = $this->getRichSnippets();
            $itemRefData = [];
            if ($trackingSnippet['reviews_count'] > 0) {
                foreach ($trackingSnippet['reviews'] as $key => $aggregateReviewDetails) {
                    $itemRefData[] = 'tb-review-' . $key;
                }
                $itemRef = implode(' ', $itemRefData);
            }

            return $itemRef;
        } catch (\Exception $e) {
            $this->debug('Error :' . $e);
        }

        return [];
    }

    /**
     * Get the TargetBay review count and rating for product
     *
     * @return array
     */
    public function getSiteReviewSnippets()
    {
        if (!$this->trackingEnabled()) {
            return false;
        }

        try {
            $data = [];
            $cacheResponseId = 'site_tbreview_response';
            $cacheSiteTbCountId = 'site_tbreview_count';
            $type = self::RATINGS_STATS;
            $cacheTag = 'block_html';

            if (false !== ($responseBody = $this->_cacheType->load($cacheResponseId))) {
                $responseBody = unserialize($responseBody);
            }

            if (false !== ($cacheSiteTbReviewCount = $this->_cacheType->load($cacheSiteTbCountId))) {
                $cacheSiteTbReviewCount = unserialize($cacheSiteTbReviewCount);
            }

            if (!empty($responseBody) && 
                !empty($tbrcnt = $responseBody->reviews_count) &&
                ($responseBody->reviews_count == $cacheSiteTbReviewCount)) {
                $averageScore = $responseBody->reviews_average;
                $reviewsCount = $responseBody->reviews_count;
                $reviewsDetails = $responseBody->reviews;

                $responseData = [
                    'average_score' => $averageScore,
                    'reviews_count' => $reviewsCount,
                    'reviews' => $reviewsDetails
                ];
                return $responseData;
            } else {
                $data['index_name'] = $this->getApiIndex();
                $jsonData = json_encode($data);
                $apiToken = '?api_token=' . $this->getApiToken();
                $feedUrl = $this->getHostname() . $type . $apiToken;
                $response = $this->postPageInfo($feedUrl, $jsonData);
                $responseBody = json_decode($response);
                if (!empty($responseBody) && ($responseBody->reviews_count !== '')) {
                    $this->_cacheType->remove($cacheResponseId);
                    $this->_cacheType->remove($cacheSiteTbCountId);
                    $this->_cacheType->save(serialize($responseBody->reviews), $cacheResponseId, [$cacheTag]);
                    $this->_cacheType->save(serialize($responseBody->reviews_count), $cacheSiteTbCountId);

                    $averageScore = $responseBody->reviews_average;
                    $reviewsCount = $responseBody->reviews_count;
                    $reviewsDetails = $responseBody->reviews;
                    $responseData = [
                        'average_score' => $averageScore,
                        'reviews_count' => $reviewsCount,
                        'reviews' => $reviewsDetails
                    ];
                    return $responseData;
                }
            }
        } catch (\Exception $e) {
            $this->debug('Error :' . $e);
        }

        return [];
    }

    /**
     * Get the Customer group name
     *
     * @param $customerGroupId
     *
     * @return array
     */
    public function getCustomerGroup($customerGroupId)
    {
        $collection = $this->_objectManager->get('\Magento\Customer\Model\Group')->load($customerGroupId);

        // Get current customer group name.
        return $collection->getCustomerGroupCode();
    }


    /**
    * Get order id from checkout session
    * @return int $orderId
    */
    public function getOrderId()
    {
       $orderId = $this->_checkoutSession->getLastOrderId();
       return $orderId;
    }

    /*
    *  set TB fetch points after user logged in 
    */

    public function setCookieValuesWithEncode()
    {
        if($this->_customerSession->isLoggedIn()) {

            $customerId = $this->_customerSession->getCustomerId();
            $customerName = $this->_customerSession->getCustomer()->getName();
            $customerEmail = $this->_customerSession->getCustomer()->getEmail();

            if($this->_customerSession->getId()){
                $customerSession = $this->_customerSession->getId();
            }else{
                $customerSession = $customerId;
            }
            $quoteId = "";

            if($this->_checkoutSession->getQuoteId()){
                $quoteId = $this->_checkoutSession->getQuoteId();
            }

            $cookieData = '_uid=' . $customerId;
            $cookieData .= '&_un=' . $customerName;
            $cookieData .= '&_uem=' . $customerEmail;
            $cookieData .= '&_utid=' . $customerId;
            $cookieData .= '&_usid=' . $customerId;
            $cookieData .= '&_uoid=' . $quoteId;
            $cookieData .= '&_ulogin=1';
            $cookieData .= '&_uasid=' . $customerSession;

             // set duration seconds for 1 year.
            $metadata = $this->_cookieMetadataFactory->createPublicCookieMetadata()
                        ->setDuration(31536000)
                        ->setPath('/')
                        ->setDomain($this->_coreSession->getCookieDomain())
                        ->setHttpOnly(false);

            $this->_cookieManager->setPublicCookie('tb_fetch_points',base64_encode($cookieData), $metadata);
        } 
       
    }

    /*
        set utm codes in browser cookies.
        Get the utm_source,utm_token from the GET request and store in browser cookie.
    */

    public function setUtmCodes()
    {
        $metadata = $this->_cookieMetadataFactory
            ->createPublicCookieMetadata()
            ->setDuration(self::TIMEOUT)
            ->setPath('/')
            ->setDomain($this->_coreSession->getCookieDomain())
            ->setHttpOnly(false);

        if ($this->_request->getParam('utm_source') !== '') {

                $this->_cookieManager->setPublicCookie(
                    'utm_source',
                    $this->_request->getParam('utm_source'),
                    $metadata
                );
        }

        if ($this->_request->getParam('token') !== '') {

                $this->_cookieManager->setPublicCookie(
                    'utm_token',
                    $this->_request->getParam('token'),
                    $metadata
                );
        }
    }

}
