<?php

namespace Targetbay\Tracking\Helper;

use Magento\Framework\App\RequestInterface as RequestInterface;

/**
 * Class Inventory
 *
 * Custom helper class
 *
 * @package Targetbay\Tracking\Helper
 */
class Inventory extends \Magento\Framework\App\Helper\AbstractHelper
{
    /**
     * @var \Magento\Framework\Registry
     */
    protected $registry;

    /**
     * @var \Magento\CatalogInventory\Model\Stock\StockItemRepository
     */
    protected $stockItemRepository;

    /**
     * @var \Magento\Store\Model\StoreManagerInterface
     */
    protected $storeManager;

    /**
     * @var \Magento\Catalog\Model\Product
     */
    protected $productModel;

    /**
     * @var \Magento\Framework\ObjectManagerInterface
     */
    protected $objectManager;

    /**
     * @var RequestInterface
     */
    public $request;

    /**
     * @param \Magento\Framework\Registry $registry
     * @param \Magento\Store\Model\StoreManagerInterface $storeManager
     * @param \Magento\Catalog\Model\Product $productModel
     * @param \Magento\Framework\ObjectManagerInterface $objectManager
     * @param RequestInterface $request
     */
    public function __construct(
        \Magento\Framework\Registry $registry,
        \Magento\Store\Model\StoreManagerInterface $storeManager,
        \Magento\Catalog\Model\Product $productModel,
        \Magento\Framework\ObjectManagerInterface $objectManager,
        RequestInterface $request
    ) {
        $this->registry = $registry;
        $this->productModel = $productModel;
        $this->storeManager = $storeManager;
        $this->objectManager = $objectManager;
        $this->_request = $request;
    }

    public function getProduct()
    {
	$product = $this->registry->registry('current_product');
        if(!$product){
            $pId = $this->_request->getParam('id');
            $product = $this->productModel->load($pId);
        }
        return $product;
    }

    public function getStockItem($productId)
    {
        $productStockObj = $this->objectManager->get('Magento\CatalogInventory\Api\StockRegistryInterface')
                        ->getStockItem($productId);
        return $productStockObj;
    }

    public function getProductInfo()
    {
        $product = $this->getProduct();
        $currentProductId = $product->getId();
        $currentProductTypeId = $product->getTypeId();
        $storeId = $this->storeManager->getStore()->getId();
        $data =  [];
        $childProductIds = [];

        switch ($currentProductTypeId) {
            case \Magento\ConfigurableProduct\Model\Product\Type\Configurable::TYPE_CODE:
                $productTypeInstance = $product->getTypeInstance();
                $productTypeInstance->setStoreFilter($storeId, $product);
                $childProducts = $productTypeInstance->getUsedProducts($product);
                $i=0;
                foreach ($childProducts as $child) {
                    $productStock = $this->getStockItem($child->getId());
                    if ($productStock->getIsInStock() == 0) {
                        $childProductIds[] = $child->getId();
                    }
                }
                foreach ($childProductIds as $key => $value) {
                    $data[$i]['child_product_id'] = $value;
                    $options = $this->getConfigurableProductOption($currentProductId, $value);
                    $data[$i]['options'] = $options;
                    $i++;
                }
                break;
            case \Magento\GroupedProduct\Model\Product\Type\Grouped::TYPE_CODE:
                $productTypeInstance = $product->getTypeInstance();
                $productTypeInstance->setStoreFilter($storeId, $product);
                $childProducts = $productTypeInstance->getAssociatedProducts($product);
                foreach ($childProducts as $child) {
                    $productStock = $this->getStockItem($child->getId());
                    if ($productStock->getIsInStock() == 0) {
                        $childProductIds[] = $child->getId();
                    }
                }
                $i=0;
                foreach ($childProductIds as $key => $value) {
                    $data[$i]['child_product_id'] = $value;
                    $childProductName = $this->getProductNameOption($value);
                    $data[$i]['options'] = $childProductName;
                    $i++;
                }
                break;
            case \Magento\Catalog\Model\Product\Type::TYPE_BUNDLE:
                $productTypeInstance = $product->getTypeInstance();
                $productTypeInstance->setStoreFilter($storeId, $product);
                $selectionCollection = $productTypeInstance->getSelectionsCollection(
                    $product->getTypeInstance(true)->getOptionsIds($product),
                    $product
                );

                $i=0;
                foreach ($selectionCollection as $child) {
                    $productStock = $this->getStockItem($child->getId());
                    if ($productStock->getIsInStock() == 0) {
                        $childProductIds[] = $child->getId();
                    }
                }
                foreach ($childProductIds as $key => $value) {
                    $data[$i]['child_product_id'] = $value;
                    $childProductName = $this->getProductNameOption($value);
                    $data[$i]['options'] = $childProductName;
                    $i++;
                }
                break;
            case \Magento\Catalog\Model\Product\Type::TYPE_SIMPLE:
                $data['child_product_id'] = '';
                $data['options'] = '';
                break;
            default:
                break;
        }
        return json_encode($data);
    }

    public function getProductNameOption($childProductId)
    {
        $productData =  [];
        $quotes = ["'", '"'];
        $replace = ['', ''];
        $storeId = $this->storeManager->getStore()->getId();
        $childProduct = $this->productModel->setStoreId($storeId)->load($childProductId);
        $productName = $childProduct->getName();
        $productData['name'] = str_replace($quotes, $replace, $productName);
        return $productData;
    }

    public function getConfigurableProductOption($parentProductId, $childProductId)
    {
        $storeId = $this->storeManager->getStore()->getId();
        $parentProduct = $this->productModel->setStoreId($storeId)->load($parentProductId);
        $attributes = $parentProduct->getTypeInstance(true)->getConfigurableAttributesAsArray(
            $parentProduct
        );
        $childProduct = $this->productModel->load($childProductId);
        $optionLabel = [];
        foreach ($attributes as $key => $attribute) {
            foreach ($attribute['values'] as $value) {
                $childValue = $childProduct->getData($attribute['attribute_code']);
                if ($value['value_index'] == $childValue) {
                    $optionLabel[] = $attribute['store_label'] . ':' . $value['store_label'];
                }
            }
        }
        return $optionLabel;
    }

    public function getConfigurableProductOptionIds($parentProductId, $childProductId)
    {
        $storeId = $this->storeManager->getStore()->getId();
        $parentProduct = $this->productModel->setStoreId($storeId)->load($parentProductId);
        $attributes = $parentProduct->getTypeInstance(true)->getConfigurableAttributesAsArray(
            $parentProduct
        );
        $childProduct = $this->productModel->setStoreId($storeId)->load($childProductId);

        $optionValue = [];
        foreach ($attributes as $key => $attribute) {
            foreach ($attribute['values'] as $value) {
                $childValue = $childProduct->getData($attribute['attribute_code']);
                if ($value['value_index'] == $childValue) {
                    $attributeId = $attribute['attribute_id'];
                    $optionValue[$attributeId] = $value['value_index'];
                }
            }
        }
        return $optionValue;
    }

    public function getBundledProductOptionIds($parentProductId, $childProductId)
    {
        $storeId = $this->storeManager->getStore()->getId();
        $bundledProduct = $this->productModel->setStoreId($storeId)->load($parentProductId);
        $selectionCollection = $bundledProduct->getTypeInstance(true)->getSelectionsCollection(
            $bundledProduct->getTypeInstance(true)->getOptionsIds($bundledProduct),
            $bundledProduct
        );
        $bundledItems = [];
        foreach ($selectionCollection as $option) {
            $entityId = $option->getEntityId();
            if (in_array($entityId, $childProductId)) {
                $bundleOptionId = $option->getOptionId();
                $bundledItems[$bundleOptionId] = $option->getSelectionId();
            }
        }
        return $bundledItems;
    }

    public function getInventryStatus()
    {
        $product = $this->getProduct();
        $currentProductId = $product->getId();
        $currentProductTypeId = $product->getTypeId();
        $storeId = $this->storeManager->getStore()->getId();
        $status = 0;
        $stockstatus = [];
        switch ($currentProductTypeId) {
            case \Magento\ConfigurableProduct\Model\Product\Type\Configurable::TYPE_CODE:
                $productTypeInstance = $product->getTypeInstance();
                $productTypeInstance->setStoreFilter($storeId, $product);
                $childProducts = $productTypeInstance->getUsedProducts($product);
                foreach ($childProducts as $child) {
                    $productStock = $this->getStockItem($child->getId());
                    if ($productStock->getIsInStock() == 0) {
                        $stockstatus[] = $child->getId();
                    }
                }
                if (count($stockstatus) > 0) {
                    $status = true;
                }
                break;
            case \Magento\GroupedProduct\Model\Product\Type\Grouped::TYPE_CODE:
                $productTypeInstance = $product->getTypeInstance();
                $productTypeInstance->setStoreFilter($storeId, $product);
                $childProducts = $productTypeInstance->getAssociatedProducts($product);
                foreach ($childProducts as $child) {
                    $productStock = $this->getStockItem($child->getId());
                    if ($productStock->getIsInStock() == 0) {
                        $stockstatus[] = $child->getId();
                    }
                }
                if (count($stockstatus) > 0) {
                    $status = true;
                }
                break;
            case \Magento\Catalog\Model\Product\Type::TYPE_BUNDLE:

                $productTypeInstance = $product->getTypeInstance(true);
                $productTypeInstance->setStoreFilter($storeId, $product);
                $selectionCollection = $productTypeInstance->getSelectionsCollection(
                    $product->getTypeInstance(true)->getOptionsIds($product),
                    $product
                    );

                foreach ($selectionCollection as $child) {
                    $productStock = $this->getStockItem($child->getId());
                    if ($productStock->getIsInStock() == 0) {
                        $stockstatus[] = $child->getId();
                    }
                }
                if (count($stockstatus) > 0) {
                    $status = true;
                }
                break;
            case \Magento\Catalog\Model\Product\Type::TYPE_SIMPLE:
                $productStock = $this->getStockItem($currentProductId);
                if ($productStock->getIsInStock() == 0) {
                    $status =  true;
                }
                break;
            default:
                break;
        }
        return $status;
    }
}
