<?php

/**
 * @author Targetbay Team
 * @copyright Copyright (c) 2016 Targetbay
 * @package Targetbay_Tracking
 */

namespace Targetbay\Tracking\Model;

use Targetbay\Tracking\Api\CartItemRepoInterface;

/**
 * Defines the implementation class of the CartItemRepo.
 */
class CartItemRepo implements CartItemRepoInterface
{

    /**
     * @var \Targetbay\Tracking\Helper\Data $trackingHelper
     */
    public $trackingHelper;

    /**
     * @var \Magento\Quote\Model\Quote
     */
    protected $quoteFactory;

    /**
     * @var \Magento\Framework\App\ResourceConnection
     */
    protected $resourceConnection;

    /**
     * @var \Magento\Quote\Model\ResourceModel\Quote\Collection
     */
    protected $quoteCollection;

    public function __construct(
        \Magento\Quote\Model\Quote $quoteFactory,
        \Magento\Framework\App\ResourceConnection $resourceConnection,
        \Magento\Quote\Model\ResourceModel\Quote\Collection $quoteCollection
    ) {
        $this->_quoteFactory = $quoteFactory;
        $this->_resourceConnection = $resourceConnection;
        $this->_quoteCollection = $quoteCollection;
    }

    /**
     * get customer cart items.
     *
     * @return CartItemRepoInterface[]
     */
    public function getList(\Magento\Framework\Api\SearchCriteria $searchCriteria)
    {
        $quoteCollection = $this->getQuoteCollectionQuery($searchCriteria);
        $cartItems = [];

        foreach ($quoteCollection as $id => $quoteInfo) {
            $quoteId = $quoteInfo['entity_id'];
            $quote = $this->_quoteFactory->setId($quoteId);
            $items = $quote->getAllVisibleItems();
            if (count($items) > 0) {
                $cartItems[$id]['entity_id'] = $quoteInfo['entity_id'];
                $cartItems[$id]['customer_id'] = $quoteInfo['customer_id'];
                $cartItems[$id]['customer_email'] = $quoteInfo['customer_email'];
                $cartItems[$id]['abandonded_at'] = $quoteInfo['updated_at'];
                $cartItems[$id]['cart_items'] = $this->_trackingHelper->getQuoteItems($items);
            }
        }

        return $cartItems;
    }

    /**
     * get quote collection.
     *
     * @return CartItemRepoInterface[]
     */
    public function getQuoteCollectionQuery($searchCriteria)
    {
        $quoteTable = $this->_resourceConnection->getTableName('quote_item');

        $quoteObj = $this->_quoteCollection->addFieldToSelect([
            'customer_id',
            'customer_firstname',
            'customer_lastname',
            'customer_email',
            'updated_at'])
            ->addFieldToFilter('customer_email', ['neq' => ''])
            ->addFieldToFilter('customer_id', ['neq' => '']);

        $quoteObj->getSelect()->join(
            ['Q2' => $quoteTable],
            '`main_table`.`entity_id` = `Q2`.`quote_id`',
            ['*']
        )->group('Q2.quote_id');

        $quoteObj->setCurPage($searchCriteria->getCurrentPage());
        $quoteObj->setPageSize($searchCriteria->getPageSize());
        return $quoteObj;
    }
}
