<?php

/**
 * @author Targetbay Team
 * @copyright Copyright (c) 2016 Targetbay
 * @package Targetbay_Tracking
 */

namespace Targetbay\Tracking\Model;

use Targetbay\Tracking\Api\CreateSubscriptionInterface;

/**
 * Defines the implementation class of the CreateSubscription.
 */
class CreateSubscription implements CreateSubscriptionInterface
{
    const STATUS_SUBSCRIBED = 1;

    /**
     * @var \Magento\Framework\App\RequestInterface
     */
    protected $request;

    /**
     * @var \Targetbay\Tracking\Helper\Data
     */
    protected $trackingHelper;

    /**
     * @var \Magento\Store\Model\StoreManagerInterface
     */
    protected $storeManager;

    /**
     * @var \Magento\Newsletter\Model\SubscriberFactory
     */
    protected $subscriberFactory;
    /**
     * @var \Magento\Newsletter\Model\Subscriber
     */
    protected $subscriberModel;

    public function __construct(
        \Magento\Framework\App\RequestInterface $request,
        \Targetbay\Tracking\Helper\Data $trackingHelper,
        \Magento\Store\Model\StoreManagerInterface $storeManager,
        \Magento\Customer\Model\Session $customerSession,
        \Magento\Customer\Api\AccountManagementInterface $customerAccountManagement,
        \Magento\Newsletter\Model\SubscriberFactory $subscriberFactory,
        \Magento\Newsletter\Model\Subscriber $subscriberModel
    ) {
        $this->_request = $request;
        $this->_trackingHelper = $trackingHelper;
        $this->_storeManager = $storeManager;
        $this->_subscriberFactory = $subscriberFactory;
        $this->_customerSession = $customerSession;
        $this->_customerAccountManagement = $customerAccountManagement;
        $this->_subscriberModel = $subscriberModel;
    }

    /**
     * create new newsletter subscription.
     * @throws \Exception
     * @return int
     */
    public function createsubscription()
    {
        $email = $this->_request->getParam('email');
        $message = [];

        $message['message'] = $this->validateEmailFormat($email);
        $message['message'] = $this->validateEmailAvailable($email);
        $message['message'] = $this->validateAlreadySuscribed($email);
        try {
            if (isset($message)) {
                $status = $this->_subscriberFactory->create()->subscribe($email);
                if ($status) {
                    $message['message'] = 'Thank you for your subscription.';
                }
            }
            return $message;
        } catch (\Exception $e) {
            $this->_trackingHelper->debug($e->getMessage());
        }
    }

    /**
     * Validates that the email address isn't being used by a different account.
     *
     * @param string $email
     * @throws \Magento\Framework\Exception\LocalizedException
     * @return void
     */
    protected function validateEmailAvailable($email)
    {
        $websiteId = $this->_storeManager->getStore()->getWebsiteId();
        if ($this->_customerSession->getCustomerDataObject()->getEmail() !== $email
            && !$this->_customerAccountManagement->isEmailAvailable($email)
        ) {
            throw new \Magento\Framework\Exception\LocalizedException(
                __('This email address is already assigned to another user.')
            );
        }
    }

    /**
     * Validates that the email address already subscribed
     *
     * @param string $email
     * @throws \Magento\Framework\Exception\LocalizedException
     * @return void
     */
    protected function validateAlreadySuscribed($email)
    {
        $subscribedUser = $this->_subscriberModel->loadByEmail($email);

        if ($subscribedUser->isSubscribed()) {
            throw new \Magento\Framework\Exception\LocalizedException(
                __('This email address is already subscribed.')
            );
        }
    }

    /**
     * Validates the format of the email address
     *
     * @param string $email
     * @throws \Magento\Framework\Exception\LocalizedException
     * @return void
     */
    protected function validateEmailFormat($email)
    {
        if (!\Zend_Validate::is($email, 'EmailAddress')) {
            throw new \Magento\Framework\Exception\LocalizedException(__('Please enter a valid email address.'));
        }
    }
}
