<?php

/**
 * @author Targetbay Team
 * @copyright Copyright (c) 2016 Targetbay
 * @package Targetbay_Tracking
 */

namespace Targetbay\Tracking\Model;

use Targetbay\Tracking\Api\CustomerRepoInterface;

/**
 * Defines the implementation class of the CustomerRepoInterface.
 */
class CustomerRepo implements CustomerRepoInterface
{
    /**
     * @var \Targetbay\Tracking\Helper\Data $trackingHelper
     */
    public $trackingHelper;

    /**
     * @var \Magento\Customer\Model\CustomerFactory
     */
    protected $customerFactory;

    protected $request;

    /**
     * @param \Targetbay\Tracking\Helper\Data $trackingHelper
     * @param \Magento\Customer\Model\CustomerFactory $customerFactory
     */
    public function __construct(
        \Targetbay\Tracking\Helper\Data $trackingHelper,
        \Magento\Customer\Model\CustomerFactory $customerFactory,
        \Magento\Framework\App\RequestInterface $request
    ) {
        $this->_trackingHelper  = $trackingHelper;
        $this->_customerFactory = $customerFactory;
        $this->request = $request;
    }

    /**
     * Get customer list
     * @return \Magento\Customer\Api\Data\CustomerSearchResultsInterface
     */
    public function getList(\Magento\Framework\Api\SearchCriteria $searchCriteria)
    {
        $storeId = $this->request->getParam('store_id');
        $collection = $this->_customerFactory->create()->getCollection();
        $collection->addFieldToFilter('store_id', $storeId);

        $collection->addNameToSelect();

        $collection->joinAttribute(
            'billing_postcode',
            'customer_address/postcode',
            'default_billing',
            null,
            'left'
        )
            ->joinAttribute('billing_city', 'customer_address/city', 'default_billing', null, 'left')
            ->joinAttribute(
                'billing_telephone',
                'customer_address/telephone',
                'default_billing',
                null,
                'left'
            )
            ->joinAttribute('billing_region', 'customer_address/region', 'default_billing', null, 'left')
            ->joinAttribute(
                'billing_country_id',
                'customer_address/country_id',
                'default_billing',
                null,
                'left'
            )
            ->joinAttribute('company', 'customer_address/company', 'default_billing', null, 'left');

        $collection->setCurPage($searchCriteria->getCurrentPage());
        $collection->setPageSize($searchCriteria->getPageSize());
        
        //$customer = $collection->load()->toArray();

        foreach ($collection as $id => $data) {
            $customer[$id] = $data->toArray();
            //$status = $this->_trackingHelper->getSubscriptionStatus($data->getEmail());
            //$customer[$id]['subscription_status'] = $status;
            $customer[$id]['group_name'] = $this->_trackingHelper->getCustomerGroup($data->getGroupId());
        }

        return $customer;
    }
}
