<?php

/**
 * @author Targetbay Team
 * @copyright Copyright (c) 2016 Targetbay
 * @package Targetbay_Tracking
 */

namespace Targetbay\Tracking\Model;

use Magento\Framework\App\ResourceConnection as Connection;
use Magento\Quote\Model\ResourceModel\Quote\Collection as QuoteCollection;
use Targetbay\Tracking\Api\TotalCartItemInterface;

/**
 * Defines the implementation class of the TotalCartItem.
 */
class TotalCartItem implements TotalCartItemInterface
{
    /**
     * @var Connection
     */
    protected $connection;

    /**
     * @var QuoteCollection
     */
    protected $quoteCollection;

    /**
     * @param \Magento\Framework\Model\Context $context
     */
    public function __construct(
        Connection $connection,
        QuoteCollection $quoteCollection
    ) {
        $this->_connection = $connection;
        $this->_quoteCollection = $quoteCollection;
    }

    /**
     * Get the Total Cart item
     *
     * @return totals
     */
    public function totalcartitemcount()
    {
        $quoteTable = $this->_connection->getTableName('quote_item');

        $quoteCollectionObj = $this->_quoteCollection
            ->addFieldToSelect([
                'customer_id',
                'customer_firstname',
                'customer_lastname',
                'customer_email',
                'updated_at'])
            ->addFieldToFilter('customer_email', ['neq' => ''])
            ->addFieldToFilter('customer_id', ['neq' => '']);

        $quoteCollectionObj->getSelect()->join(
            ['Q2' => $quoteTable],
            '`main_table`.`entity_id` = `Q2`.`quote_id`',
            ['*']
        )->group('Q2.quote_id');

        $totals = [
            'total_cartitem' => $quoteCollectionObj->getSize()
        ];

        return json_encode($totals);
    }
}
