<?php

namespace Targetbay\Tracking\Observer;

use Magento\Framework\App\RequestInterface as RequestInterface;
use Magento\Framework\Escaper as EscaperObject;
use Magento\Framework\Event\Observer;
use Magento\Framework\Event\ObserverInterface;

class AddCartEventHandler implements ObserverInterface {
	const ADD_TO_CART = 'add-to-cart';

	protected $_cart;
	protected $_trackingHelper;
	protected $_postdataHelper;
	protected $_objectManager;
	protected $_request;
	protected $_escapeObj;
	protected $_productloader;

	public function __construct(\Targetbay\Tracking\Helper\Data $trackingHelper, \Magento\Checkout\Model\Cart $cart, \Magento\Framework\ObjectManagerInterface $objectManager, EscaperObject $escapeObj, RequestInterface $request, \Targetbay\Tracking\Helper\Postdata $postdataHelper, \Magento\Catalog\Model\ProductFactory $productFactory) {
		$this -> _cart = $cart;
		$this -> _trackingHelper = $trackingHelper;
		$this -> _postdataHelper = $postdataHelper;
		$this -> _objectManager = $objectManager;
		$this -> _request = $request;
		$this -> _escapeObj = $escapeObj;
		$this -> _productloader = $productFactory;
	}

	/**
	 * @param Observer $observer
	 * @return bool
	 */
	public function execute(Observer $observer) {
		if (!$this -> _trackingHelper -> canTrackPages(self::ADD_TO_CART)) {
			return false;
		}
		$productInfo = $observer -> getProduct();

		$productEventInfo = $observer -> getEvent() -> getProduct();
		$productQty = $productEventInfo -> getQty();
		$quote = $this -> _cart -> getQuote();
		$item = $quote -> getItemByProduct($productEventInfo);
		if ($productInfo -> getId()) {
			try {
				switch ($productInfo->getData('type_id')) {
					case 'grouped' :
						$productIds = $this -> _request -> getParam('super_group');

						$dataItem = [];
						$productData = [];
						foreach ($productIds as $id => $qty) {
							if ($qty < 1) {
								continue;
							}
							$product = $this -> _productloader -> create() -> load($id);
							$productData['type'] = $product -> getTypeId();
							$productData['parent_id'] = $this -> _request -> getParam('product');
							$productData['product_id'] = $id;
							$productData['product_sku'] = $product -> getSku();
							$productData['product_name'] = $this -> _escapeObj -> escapeHtml($product -> getName());
							$productData['price'] = $product -> getFinalPrice() * $qty;
							$productData['special_price'] = $product -> getSpecialPrice();
							$productData['productimg'] = $this -> _trackingHelper -> getImageUrl($product, 'image');
							$productData['category'] = $this -> _trackingHelper -> getProductCategory($product);
							$productData['category_name'] = $this -> _trackingHelper -> getProductCategoryName($product);
							$productData['quantity'] = $qty;
							$productData['page_url'] = $product -> getUrlModel() -> getUrl($product);
							$productData['attributes'] = '';
							$dataItem[] = $productData;
						}

						$data = $this -> _trackingHelper -> getCartInfo();
						$data['product_type'] = $productInfo -> getData('type_id');
						$data['cart_item'] = $dataItem;
						break;
					default :
						$data = array_merge($this -> _trackingHelper -> getCartInfo(), $this -> _trackingHelper -> getItemInfo($item, self::ADD_TO_CART, $productQty));
						$data['price'] = $item -> getProduct() -> getFinalPrice();
						if ($customOptions = $this -> _trackingHelper -> getCustomOptionsInfo($item, null)) {
							$data['attributes'] = $customOptions;
						}
				}
				$this -> _postdataHelper -> pushPages($data, self::ADD_TO_CART);
			} catch (\Exception $e) {
				$this -> _trackingHelper -> debug("Add to cart ERROR:" . $e -> getMessage());
			}
		}
	}

}
