<?php

namespace Targetbay\Tracking\Observer;

use Magento\Framework\App\RequestInterface;
use Magento\Framework\Event\ObserverInterface;

class AddProductEventHandler implements ObserverInterface
{
    const ADD_PRODUCT = 'add-product';
    const UPDATE_PRODUCT = 'update-product';

    public $_request;
    protected $_trackingHelper;
    protected $_postdataHelper;
    protected $state;

    public function __construct(
        RequestInterface $request,
        \Targetbay\Tracking\Helper\Data $trackingHelper,
        \Targetbay\Tracking\Helper\Postdata $postdataHelper,
        \Magento\Framework\App\State $state
    ) {
        $this->_request = $request;
        $this->_trackingHelper  = $trackingHelper;
        $this->_postdataHelper = $postdataHelper;
        $this->_state = $state;
    }

   
    public function execute(\Magento\Framework\Event\Observer $observer)
    {
        if (!$this->_trackingHelper->canTrackPages(self::ADD_PRODUCT)) {
            return false;
        }

        $product = $observer->getEvent()->getProduct();

        $actionName = $this->_state->getAreaCode();

        if($actionName == 'adminhtml'){
            try {
                if ($product->getId()) {
                    $type = self::ADD_PRODUCT;
                    if ($this->_request->getParam('id')) {
                        if (!$this->_trackingHelper->canTrackPages(self::UPDATE_PRODUCT)) {
                            return false;
                        }
                        $type = self::UPDATE_PRODUCT;
                    }
                    $data = $this->_trackingHelper->getProductData($product);
                    $data['action'] = $actionName;

                    $this->_postdataHelper->pushPages($data, $type);
                }
            }catch (\Exception $e) {
                $this->_trackingHelper->debug(" '$type' ERROR:" . $e -> getMessage());
            }
        } 
    }
}
