<?php

namespace Targetbay\Tracking\Observer;

use Magento\Framework\Event\ObserverInterface;

class CreateOrderEventHandler implements ObserverInterface
{
    const ORDER_ITEMS = 'm2-ordered-items';

    public $trackingHelper;
    public $registry;
    protected $_postdataHelper;

    public function __construct(
        \Targetbay\Tracking\Helper\Data $trackingHelper,
        \Magento\Framework\Registry $registry,
        \Targetbay\Tracking\Helper\Postdata $postdataHelper
    ) {
        $this->_trackingHelper  = $trackingHelper;
        $this->_postdataHelper  = $postdataHelper;
        $this->_registry = $registry;
    }

    public function execute(\Magento\Framework\Event\Observer $observer)
    {
        $orderInfo = [];

        if (!$this->_registry->registry('order_pushed')) {

            $this->_registry->register('order_pushed', true);
            if (!$this->_trackingHelper->canTrackPages(self::ORDER_ITEMS)) {
                return false;
            }

            try {

                $orderInfo = $this->_trackingHelper->getSessionInfo();
                // Get orderId from the event.
                $orderData = $observer->getEvent()->getData('order_ids');
                foreach($orderData as $orderId){
                    $orderInfo['order_id'] = $orderId;
                }
                // call create account api call, if the user choose the register as account in checkout page.
                if ($customer = $this->_trackingHelper->isRegisterCheckout($order)) {
                    $this->pushRegisterData($customer);
                    $orderInfo['registercheckout'] = true;
                } 

                $this->_postdataHelper->pushPages($orderInfo, self::ORDER_ITEMS);
            } catch (\Exception $e) {
                $this->_trackingHelper->debug(" Placed order  ERROR:" . $e->getMessage());
            }
        }

        return true;
    }

    public function pushRegisterData($customer)
    {
        if (! $this->_trackingHelper->canTrackPages(self::CREATE_ACCOUNT)) {
            return false;
        }
        try {
            $data = $this->_trackingHelper->getCustomerData($customer, self::CREATE_ACCOUNT);
            $this->_postdataHelper->pushPages($data, self::CREATE_ACCOUNT);
        } catch (\Exception $e) {
            $this->_trackingHelper->debug("Placed order  ERROR:" . $e->getMessage());
        }
        return true;
    }
}
