<?php

namespace Targetbay\Tracking\Observer;

use Magento\Checkout\Model\Session as CheckoutSession;
use Magento\Customer\Model\Session as CustomerSession;
use Magento\Framework\Event\ObserverInterface;

class SaveAccountDataEventHandler implements ObserverInterface
{
    const CUSTOMER_ACCOUNT = 'change-user-account-info';

    protected $_trackingHelper;
    protected $_checkoutSession;
    protected $_customerSession;
    public $_date;
    protected $customer;
    protected $_postdataHelper;

    public function __construct(
        \Magento\Framework\Stdlib\DateTime\DateTime $date,
        CheckoutSession $checkoutSession,
        CustomerSession $customerSession,
        \Targetbay\Tracking\Helper\Data $trackingHelper,
        \Magento\Customer\Model\Customer  $customer,
        \Targetbay\Tracking\Helper\Postdata $postdataHelper
    ) {
        $this->_trackingHelper = $trackingHelper;
        $this->_checkoutSession = $checkoutSession;
        $this->_customerSession = $customerSession;
        $this->_date = $date;
        $this->customer = $customer;
        $this->_postdataHelper = $postdataHelper;
    }


    /**
     * Update account info observer
     *
     * @param \Magento\Framework\Event\Observer $observer
     *
     * @return void|bool
     */
    public function execute(\Magento\Framework\Event\Observer $observer)
    {
        if (!$this->_trackingHelper->canTrackPages(self::CUSTOMER_ACCOUNT)) {
            return false;
        }
        
        try {
            $this->_checkoutSession->setTitle('Account Information');
            $data = $this->_trackingHelper->visitInfo();
            $customerId = $this->_customerSession->getCustomerId();
            // $customerObj = $this->_objectManager->get('Magento\Customer\Model\Customer');
            $data['customer_id'] = $customerId;
            $data['firstname'] = $this->customer->getFirstname();
            $data['lastname'] = $this->customer->getLastname();
            $data['email'] = $this->customer->getEmail();
            $data['account_updated'] = $this->_date->date('Y-m-d');
            $this->_postdataHelper->pushPages($data, self::CUSTOMER_ACCOUNT);
        }catch (\Exception $e) {
            $this->_trackingHelper->debug("save customer account data ERROR:" . $e -> getMessage());
        }
    }
}
