<?php

namespace Targetbay\Tracking\Observer;

use Magento\Customer\Model\Session as CustomerSession;
use Magento\Framework\Event\ObserverInterface;

class SubscribeCustomerEventHandler implements ObserverInterface
{
    const SUBSCRIBE_CUSTOMER = 'user-subscribe';

    const STATUS_SUBSCRIBED = 1;

    const STATUS_NOT_ACTIVE = 2;

    const STATUS_UNSUBSCRIBED = 3;

    const STATUS_UNCONFIRMED = 4;

    public $_trackingHelper;

    public $_customerSession;

    public $_request;

    public $_storeManager;

    public $_objectManager;

    protected $_subscriber;

    public function __construct(
        CustomerSession $customerSession,
        \Targetbay\Tracking\Helper\Data $trackingHelper,
        \Magento\Store\Model\StoreManagerInterface $storeManager,
        \Magento\Framework\App\RequestInterface $request,
        \Magento\Framework\ObjectManagerInterface $objectManager,
        \Magento\Newsletter\Model\Subscriber $subscriber,
        \Targetbay\Tracking\Helper\Postdata $postdataHelper

    ) {
        $this->_trackingHelper = $trackingHelper;
        $this->_customerSession = $customerSession;
        $this->_storeManager = $storeManager;
        $this->_objectManager = $objectManager;
        $this->_subscriber= $subscriber;
        $this->_request = $request;
        $this->_postdataHelper = $postdataHelper;
    }


    /**
     * @param \Magento\Framework\Event\Observer $observer
     * @return bool|void
     */
    public function execute(\Magento\Framework\Event\Observer $observer)
    {
        if (!$this->_trackingHelper->canTrackPages(self::SUBSCRIBE_CUSTOMER)) {
            return false;
        }
        
        try {
            $websiteId = $this->_storeManager->getStore()->getWebsiteId();
            $customerModel = $this->_objectManager->get('Magento\Customer\Model\Customer');

            $this->_customerSession->setTitle('Newsletter Subscription');

            if ($this->_request->getParam('email')) {
                $email = $this->_request->getParam('email');
                $customer = $customerModel->setWebsiteId($websiteId)->loadByEmail($email);
                $customerId = $customer->getEntityId();
            } else {
                $customerId = $this->_customerSession->getCustomer()->getId();
                $email = $this->_customerSession->getCustomer()->getEmail();
            }

            $data = $this->_trackingHelper->visitInfo();

            if (!empty($customerId)) {
                $subscriberFactory = $this->_subscriber->loadByEmail($email);

                switch ($subscriberFactory->getSubscriberStatus()) {
                    case self::STATUS_UNSUBSCRIBED:
                        $status = 'Unsubscribed';
                        break;
                    case self::STATUS_SUBSCRIBED:
                        $status = 'Subscribed';
                        break;
                    case self::STATUS_UNCONFIRMED:
                        $status = 'Unconfirmed';
                        break;
                    case self::STATUS_NOT_ACTIVE:
                        $status = 'Not Activated';
                        break;
                    default:
                        $status = $subscriberFactory->getSubscriberStatus();
                        break;
                }
            } else {
                $status = '';
            }

            $status = !empty($email) ? 'Subscribed' : $status;
            $data['user_mail'] = $this->_customerSession->isLoggedIn() ?
                                    $this->_customerSession->getCustomer()->getEmail() : $email;
            $data['subscription_status'] = $status;
            $this->_postdataHelper->pushPages($data, self::SUBSCRIBE_CUSTOMER);
        } catch (\Exception $e) {
             $this->_trackingHelper->debug("subscription ERROR:" . $e -> getMessage());
        }
    }
}
