<?php

namespace Targetbay\Tracking\Observer;

use Magento\Checkout\Model\Session as CheckoutSession;
use Magento\Framework\Event\ObserverInterface;

class UpdateCartEventHandler implements ObserverInterface
{
    const UPDATECART = 'update-cart';

    protected $_trackingHelper;

    protected $_checkoutSession;

    protected $_postdataHelper;

    public function __construct(
        \Targetbay\Tracking\Helper\Data $trackingHelper,
        CheckoutSession $checkoutSession,
        \Targetbay\Tracking\Helper\Postdata $postdataHelper
    ) {
        $this->_trackingHelper  = $trackingHelper;
        $this->_postdataHelper  = $postdataHelper;
        $this->_checkoutSession = $checkoutSession;
    }

    /**
     * Capture the Update cart event
     *
     * @param \Magento\Framework\Event\Observer $observer
     *
     * @return bool|void
     */
    public function execute(\Magento\Framework\Event\Observer $observer)
    {
        if (!$this->_trackingHelper->canTrackPages(self::UPDATECART)) {
            return false;
        }

        try {
            $items = $this->_checkoutSession->getQuote()->getAllVisibleItems();
            $requestInfo = $observer->getEvent()->getInfo();
            $data = $this->_trackingHelper->getCartInfo();

            foreach ($items as $item) {
                $newQty = $requestInfo [$item->getId()] ['qty'];
                $oldQty = $item->getQty();
                if ($newQty == 0 || ($newQty == $oldQty)) {
                    continue;
                }
                $itemData = $this->_trackingHelper->getItemInfo($item);
                unset($itemData ['quantity']);
                $itemData ['old_quantity'] = $oldQty;
                $itemData ['new_quantity'] = $newQty;
                $data ['cart_items'] [] = $itemData;
            }
            if (isset($data ['cart_items'])) {
                $this->_postdataHelper->pushPages($data, self::UPDATECART);
            }
        } catch (\Exception $e) {
            $this->_trackingHelper->debug(" update cart ERROR:" . $e->getMessage());
        }
    }
}
