<?php

namespace Targetbay\Tracking\Observer;

use Magento\Framework\Event\ObserverInterface;

class UserLoginEventHandler implements ObserverInterface
{
    const LOGIN = 'login';

    protected $_trackingHelper;
    protected $_postdataHelper;

    public function __construct(
        \Targetbay\Tracking\Helper\Data $trackingHelper,
        \Targetbay\Tracking\Helper\Postdata $postdataHelper
    ) {
        $this->_trackingHelper = $trackingHelper;
        $this->_postdataHelper  = $postdataHelper;
    }
    

    /**
     * @param \Magento\Framework\Event\Observer $observer
     * @return bool|void
     */
    public function execute(\Magento\Framework\Event\Observer $observer)
    {
        if (!$this->_trackingHelper->canTrackPages(self::LOGIN)) {
            $this->_trackingHelper->debug("canTrackPages");
            return false;
        }

        if (!$observer->getCustomer()) {
            $this->_trackingHelper->debug("observer");
            return false;
        }

        try {

            // update the customer details in tb_fetch_points cookie after user logged in. 
            $this->_trackingHelper->setCookieValuesWithEncode();
            $data = $this->_trackingHelper->getCustomerData($observer->getCustomer(), self::LOGIN);
            $this->_postdataHelper->pushPages($data, self::LOGIN);
        } catch (\Exception $e) {
            $this->_trackingHelper->debug("Login ERROR:" . $e->getMessage());
        }
    }
}
