<?php
namespace Targetbay\Tracking\Plugin\Cart;

class EditItem {

	protected $_trackingHelper;

	protected $_postdataHelper;

	public function __construct(\Targetbay\Tracking\Helper\Data $trackingHelper, \Targetbay\Tracking\Helper\Postdata $postdataHelper) {
		$this -> _trackingHelper = $trackingHelper;
		$this -> _postdataHelper = $postdataHelper;
	}

	public function aroundUpdateItem($subject, $proceed, $itemId, $requestInfo, $updatingParams = null) {

		if (!$this -> _trackingHelper -> canTrackPages(\Targetbay\Tracking\Observer\UpdateCartEventHandler::UPDATECART)) {
			return $proceed($itemId, $requestInfo, $updatingParams = null);
		}
		$doAddtoCart = false;
		$quoteItem = $subject -> getQuote() -> getItemById($itemId);
		$oldQty = $quoteItem -> getQty();

		if ($preOption = $requestInfo['selected_configurable_option']) {
			$currOption = $quoteItem -> getQtyOptions();
			if (!isset($currOption[$preOption]))
				$doAddtoCart = true;
		}

		if (isset($requestInfo['bundle_option'])) {
			$typeInstance = $quoteItem -> getProduct() -> getTypeInstance();
			$orderOptions = $typeInstance -> getOrderOptions($quoteItem -> getProduct());

			$oldBundleOptions = $orderOptions['info_buyRequest']['bundle_option'];
			sort($oldBundleOptions);

			$newBundleOptions = $requestInfo['bundle_option'];
			sort($newBundleOptions);

			if ($oldBundleOptions != $newBundleOptions)
				$doAddtoCart = true;
		}

		// This will run the core updateItem function and return the updated quote item.
		$quoteItem = $proceed($itemId, $requestInfo, $updatingParams = null);

		if ($quoteItem) {
			try {
				//update cart item
				if (!$doAddtoCart) {
					$newQty = $requestInfo['qty'];
					if ($newQty == 0 || ($newQty == $oldQty))
						return $quoteItem;

					$itemData = $this -> _trackingHelper -> getItemInfo($quoteItem);
					unset($itemData['quantity']);
					$itemData['old_quantity'] = $oldQty;
					$itemData['new_quantity'] = $newQty;
					$data['cart_items'][] = $itemData;

					if (isset($data['cart_items'])) {
                                                $data = array_merge($this -> _trackingHelper -> getCartInfo(), $data);
						$this -> _postdataHelper -> pushPages($data, \Targetbay\Tracking\Observer\UpdateCartEventHandler::UPDATECART);
					}
				}
				//Add to cart
				else {
					$data = array_merge($this -> _trackingHelper -> getCartInfo(), $this -> _trackingHelper -> getItemInfo($quoteItem, \Targetbay\Tracking\Observer\AddCartEventHandler::ADD_TO_CART));
					$data['price'] = $quoteItem -> getProduct() -> getFinalPrice();
					$data['attributes'] = $this -> _trackingHelper -> getCustomOptionsInfo($quoteItem, null);
					$data['quantity'] = $requestInfo['qty'];
					$this -> _postdataHelper -> pushPages($data, \Targetbay\Tracking\Observer\AddCartEventHandler::ADD_TO_CART);
				}
			} catch(\Exception $e) {
				$this -> _trackingHelper -> debug('EDIt CART ERROR:' . $e -> getMessage());
			}
		}
		return $quoteItem;
	}

}
